define(["properties/App.Settings", "system/storage/StorageInterface","controllers/UserConfigurationController", "core/SLScreenshotUploadAPIInterface","system/DateTime", "system/graphics/GraphicsInterface"], function(Settings, StorageInterface, UserConfigurationController, SLScreenshotUploadAPIInterface, DateTime, GraphicsInterface) {
	function ScreenCaptureController(){
		var self=this;
			
		var minimumFreeDiskSpace = 100;
		var minimumStoredScreenshots = 1;
		var maxmimumStoredScreenshots = 2;
		
		// if (Settings.application.sdkVersion < 1.2)
		// {
		// 	maxmimumStoredScreenshots = 2;
		// }
		
		var screenCaptureConfiguration; 
		
		var enabled = false;
		
		var _UploadAPI;
		
		var timer;
		var startTimer;
		
		var frequency;
		var uploadingOnDemandScreenshot = false;
		var performingScheduledUpload = false;
		var tmpScreenshot;
		var lastScheduledScreenshotTime;
		
		var screenshotsCurrentlyUploading = null;
			
		self.start=function(){
			if(enabled)
			{
				if(startTimer != null && startTimer != undefined)
					clearTimeout(startTimer);
				
				var timeToNextScreenshot = 0;
				if(lastScheduledScreenshotTime!= null && lastScheduledScreenshotTime!= undefined)
					timeToNextScreenshot = (lastScheduledScreenshotTime.getTime() + frequency) - DateTime.now().getTime();
				
				//We are now due a screenshot, so take one and start the timer
				if(timeToNextScreenshot < 1)
				{
					takeScheduledScreenCapture();
					
					timer = setInterval(function() {
						takeScheduledScreenCapture();
					}, frequency);
				}else{
					//We will start in the future as we aren't ready to take a screenshot yet
					startTimer = setTimeout(function(){
						self.start();
					},timeToNextScreenshot);
					
				}				
			}
		};		
		
		
			
		self.stop=function(){
			if(timer != null && timer != undefined)
				clearInterval(timer);
		};
		
		self.initialize=function(configuration)
		{
			logger.debug("Configuring screen capture");
			screenCaptureConfiguration = configuration;
			
			if(configuration.logging.screencapture != null && configuration.logging.screencapture != undefined)
			{
				//GET ON
				enabled = configuration.logging.screencapture.on;
			}else {
				enabled = false;
			}
			
			logger.debug("Screen capture enabled: " + enabled);
			
			if(enabled)
			{
				//Get frequency
				frequency = configuration.logging.screencapture.frequency * 60000;
			}	
										
		};
			
		function deleteScreenshot(screenshot)
		{
			var dfd = $.Deferred();
			
			require(["models/ScreenshotFileList"], function(ScreenshotFileList) {
				ScreenshotFileList.remove(screenshot.created).done(function(){
					logger.debug("Screenshot removed");
					dfd.resolve();
				}).fail(function()
				{
					dfd.reject();
				});	
			});
			
			return dfd.promise();
		}
		
		function deleteScreenshots(screenshotArray)
		{
			var dfd = $.Deferred();
			
			require(["models/ScreenshotFileList"], function(ScreenshotFileList) {

				ScreenshotFileList.removeMultipleScreenshots(screenshotArray).done(function(){
					logger.debug("Screenshots removed");
					dfd.resolve();
				}).fail(function()
				{
					dfd.reject();
				});	
			});
			
			return dfd.promise();
		}

		function addScreenshotToStore(screenshotData,dateTimeTaken,tz,rotation,isOnDemandScreenshot)
		{	
			var dfd = $.Deferred();
			
			//UserConfigurationController.timeZone().id
			require(["models/ScreenshotFileList"], function(ScreenshotFileList) {
						
				ScreenshotFileList.add(dateTimeTaken.getStringRepresentation(),tz,isOnDemandScreenshot,rotation,screenshotData).done(function()
				{
					dfd.resolve();
				}).fail(function()
				{
					dfd.reject();
				});	
				
			});
			
			return dfd.promise();
		}
		
				
		function takeScheduledScreenCapture(){
			logger.info("Taking scheduled screen capture");
			
			getScreenCapture().done(function(screenshot)
			{	
				lastScheduledScreenshotTime = screenshot.date;
				
				addScreenshotToStore(screenshot.data,screenshot.date,screenshot.tz,screenshot.rotation,false).done(function(){
					logger.debug("Screenshot taken and added to store");
				}).fail(function()
				{
					logger.error("There was an enrror saving the screenshot to disk");
				});	
				
			});
		}
		

		self.takeOnDemandScreenshot = function(){
			
			getScreenCapture().done(function(screenshot)
			{
				uploadingOnDemandScreenshot = true;
				//Save the data in case we have a problem uploading the screenshot
				var screenshotUploadObject = {};
				screenshotUploadObject.data = screenshot.data;
				screenshotUploadObject.created = screenshot.date.getStringRepresentation();
				screenshotUploadObject.tz = screenshot.tz;
				screenshotUploadObject.ondemand = true;
				screenshotUploadObject.rotation = screenshot.rotation;
				tmpScreenshot = screenshotUploadObject;
				// Try to upload
				uploadScreenCapture(screenshotUploadObject);
	
			});
		};
		
		function removeOldScreenshotsIfRequired()
		{
			logger.debug("Checking to see if we need to remove some screenshots");
			
			var dfdAllComplete = $.Deferred();
			
			var dfdCountComplete = $.Deferred();
			
						
			dfdCountComplete.done(function(){
				var dfdStorageClearComplete = $.Deferred();
				
				dfdStorageClearComplete.done(function()
				{
					logger.debug("Storage checks done");
					dfdAllComplete.resolve();
				});
				
				//Now, remove them if too much space is being used
				logger.debug("Checking space used by screenshots in store to see if we need to remove some");
				removeOldScreenshotsToClearStorageIfRequired(dfdStorageClearComplete);		
			});
			
			
			//First, delete by count if too many
			logger.debug("Checking number of screenshots in store to see if we need to remove some");
			removeOldScreenshotsToReduceNumberIfRequired(dfdCountComplete);
			
			return dfdAllComplete.promise();
		}
		
		function removeOldScreenshotsToReduceNumberIfRequired(dfdCountComplete)
		{
			require(["models/ScreenshotFileList"], function(ScreenshotFileList) {
				
				ScreenshotFileList.count().done(function(count)
				{
					if(count < maxmimumStoredScreenshots)
					{
						logger.debug("Screenshot count is " + count + ".");
						dfdCountComplete.resolve(true);
					} else {
						logger.debug("Screenshot count is " + count + ", removing a screenshot.");
						ScreenshotFileList.removeFirst().done(function(){
							removeOldScreenshotsToReduceNumberIfRequired(dfdCountComplete);
						});
					}
					
				});				
			});		
		}
		
		function removeOldScreenshotsToClearStorageIfRequired(dfdStorageClearComplete)
		{
			isFreeStorageAboveMin().done(function(result){
				if(result)
					dfdStorageClearComplete.resolve(true);
				else
				{
					logger.debug("Deleting a screenshot to clear space");
					require(["models/ScreenshotFileList"], function(ScreenshotFileList) {
						
						if(ScreenshotFileList.count() == 0)
							dfdStorageClearComplete.resolve(false);
						else
							ScreenshotFileList.removeFirst().done(function(){
								removeOldScreenshotsToClearStorageIfRequired(dfdStorageClearComplete);
							});
					});				
				}	
			});
		}
		
		
		function isFreeStorageAboveMin()
		{	
			var dfd = $.Deferred();
			logger.debug('Checking if there is enough disk space to take a screenshot');
			StorageInterface.getTotalStorageAvailable()
			.done(function(space) {
				logger.debug("Space available: " + space + ", Space required: " + minimumFreeDiskSpace);
				
				var result = true;
				if(space < minimumFreeDiskSpace)
				{
					logger.debug('Less disk space than the set limit - delete a screenshots');
					result = false;
				} else{
					logger.debug('Enough disk space for screenshots');
				}
		    	dfd.resolve(result);
		    })
		    .fail(function(){
		    	dfd.reject(false);
		    });
			
			return dfd.promise();
		}
	
	
		function getScreenCapture(){
			//Check that we aren't exceeding any limits and remove screenshots if necessary
			var dfd = $.Deferred();
			
			removeOldScreenshotsIfRequired().done(function()
			{	
				logger.debug("Limits checked, taking screenshot");	
				
				var options = {};
			    options.save = false;
			    options.thumbnail = false;
				
				var signage = new Signage();
			    signage.captureScreen(function (cbObject){
			    		
			    		var screenshot = {};
			    		screenshot.data = cbObject.data;
			    		screenshot.date = DateTime.now();
			    		screenshot.tz = UserConfigurationController.timeZone().id;
						GraphicsInterface.getRotationAngle()
							.done(function(angle) {
								screenshot.rotation = angle;
								logger.debug("Returning screenshot");
								dfd.resolve(screenshot);
							});
								    	
			    	},
			    	function(cbObject)
			    	{
			    		var errorCode = cbObject.errorCode;
			        	var errorText = cbObject.errorText;
			 			logger.error("Error Code [" + errorCode + "]: " + errorText);
			 			dfd.reject(cbObject);		
			    	} , options);
			});
			
			
			return dfd.promise();
		}
		
		self.uploadAllScreenshots = function()
		{
			if(performingScheduledUpload == false)
			{
				logger.info('Uploading all screenshots');
				performingScheduledUpload = true;
				uploadNextScreenshotInQueue();
			} else {
				logger.info('Screenshot upload already in progress');
			}
		};
		
		function uploadNextScreenshotInQueue()
		{
			//Grab the first 5 screenshots		
			require(["models/ScreenshotFileList"], function(ScreenshotFileList) {				
				
				 ScreenshotFileList.getFirstXItems(1).done(function(nextThreeScreenshots){
				 	
				 	screenshotsCurrentlyUploading = nextThreeScreenshots;
				 	
					if(screenshotsCurrentlyUploading == null)
					{
						logger.info("screenshotsCurrentlyUploading is null");
					} else
						logger.info("screenshotsCurrentlyUploading length is " + screenshotsCurrentlyUploading.length);			
						
					if(screenshotsCurrentlyUploading != null && screenshotsCurrentlyUploading.length > 0)
					{
						logger.debug("Uploading " + screenshotsCurrentlyUploading.length + " screenshots.");
						uploadScreenCaptureArray(screenshotsCurrentlyUploading);	
					} else{
						logger.info('All Screenshots Uploaded');
						performingScheduledUpload = false;
					}
			
				});		
				
			
			});
			
			
		}
		
		function screenshotUploadComplete()
		{
			if(uploadingOnDemandScreenshot)
			{
				logger.debug('On demand screenshot uploaded.');
				uploadingOnDemandScreenshot = false;
				tmpScreenshot = null;
			}else
			{	
				logger.debug('Screenshots uploaded. Deleting');
			

				deleteScreenshots(screenshotsCurrentlyUploading).done(function(){
					logger.debug("Screenshots deleted, upload next");
					screenshotsCurrentlyUploading = null;
					//Upload next
					uploadNextScreenshotInQueue();
				}).fail(function(){
					//Error deleting screenshots.
					//Cancel the upload and try again next time
					performingScheduledUpload = false;
					
				});
						
			}
		}
		
		function screenshotUploadFailed()
		{
			logger.error('Screenshot upload failed.');	
			
			if(uploadingOnDemandScreenshot)
			{
				//On demand screenshot could not be uploaded. Add to the screenshot list to upload later
				uploadingOnDemandScreenshot = false;
				
				addScreenshotToStore(tmpScreenshot,tmpScreenshot.created,tmpScreenshot.tz,tmpScreenshot.rotation,true);			
			} else {
				//Currently this cancels the upload
				performingScheduledUpload = false;
			}	
		}
				
		function initializeUploadAPI()
		{
			if(_UploadAPI == null || _UploadAPI == undefined)
			{				
				_UploadAPI = new SLScreenshotUploadAPIInterface();
				_UploadAPI.initialize(screenshotUploadComplete,screenshotUploadFailed);			
			}
		}

		function uploadScreenCaptureArray(screenshotUploadArray)
		{	
			logger.debug("uploadScreenCaptureArray");
			initializeUploadAPI();
			
		
			//Upload the screenshot
			_UploadAPI.uploadMultipleScreenshots(screenshotUploadArray);	
		}

		function uploadScreenCapture(screenshotUploadObject)
		{	
			logger.debug("uploadScreenCapture");
			initializeUploadAPI();
			
			//Upload the screenshot
			_UploadAPI.uploadSingleScreenshot(screenshotUploadObject);		
		}
	
	}
	return ScreenCaptureController;
}); 