define(["models/UserConfigurationModel"], function(UserConfigurationModel) {
    /**
     * [UserConfigurationController description]
     */
    function UserConfigurationController() {
        var self = this;
        var _propertyChangedHandlers = [];
        var _userConfiguration = null;

        /**
         * [initialize description]
         * @return {[type]} [description]
         */
        self.initialize = function() {
            var dfd = $.Deferred();

            new UserConfigurationModel().initialize().done(function(config) {
                _userConfiguration = config;
                dfd.resolve();
            }).fail(function() {
                dfd.reject();
            });

            return dfd;
        };

        /**
         * [addEventListener description]
         * @param {Function} callback [description]
         */
        self.addEventListener = function(callback) {
            _propertyChangedHandlers.push(callback);
        };

        /**
         * [removeEventListener description]
         * @param  {Function} callback [description]
         * @return {[type]}            [description]
         */
        self.removeEventListener = function(callback) {

        };

        /**
         * [reload description]
         * @return {[type]} [description]
         */
        self.reload = function() {
            _userConfiguration = null;
        };

        /**
         * [hasSaved description]
         * @return {Boolean} [description]
         */
        self.hasSaved = function() {
            return getUserConfigurationModel().hasSaved;
        };

        /**
         * [volumeUp description]
         * @return {[type]} [description]
         */
        self.volumeUp = function() {
            getUserConfigurationModel().isMuted = false;

            notifyConsumers('volumeUp', null);
        };

        /**
         * [volumeDown description]
         * @return {[type]} [description]
         */
        self.volumeDown = function() {
            getUserConfigurationModel().isMuted = false;

            notifyConsumers('volumeDown', null);
        };

        /**
         * [isMuted description]
         * @return {Boolean} [description]
         */
        self.isMuted = function() {
            return getUserConfigurationModel().isMuted;
        };

        /**
         * [toggleMute description]
         * @return {[type]} [description]
         */
        self.toggleMute = function() {
            if (getUserConfigurationModel().isMuted)
                getUserConfigurationModel().isMuted = false;
            else
                getUserConfigurationModel().isMuted = true;

            notifyConsumers('mute', getUserConfigurationModel().isMuted);
        };

        /**
         * [rotation description]
         * @param  {[type]} rotation [description]
         * @return {[type]}          [description]
         */
        self.rotation = function(rotation) {
            if (rotation !== undefined) {
                getUserConfigurationModel().rotation = rotation;
                notifyConsumers('rotation', getUserConfigurationModel().rotation);
            }
            return getUserConfigurationModel().rotation;
        };

        self.rotateVideos = function(rotateVideos) {
            if (rotateVideos !== undefined) {
                getUserConfigurationModel().rotateVideos = rotateVideos;
                //getUserConfigurationModel().save();
            }
            return getUserConfigurationModel().rotateVideos;
        };

        /**
         * [isRotated description]
         * @return {Boolean} [description]
         */
        self.isRotated = function() {
            return (getUserConfigurationModel().rotation.angle != 0) ? true : false;
        };

        /**
         * [rotationAngle description]
         * @return {[type]} [description]
         */
        self.rotationAngle = function() {
            return getUserConfigurationModel().rotation.angle;
        };

        /**
         * [showTrace description]
         * @param  {[type]} showTrace [description]
         * @return {[type]}           [description]
         */
        self.showTrace = function(showTrace) {
            if (showTrace !== undefined) {
                getUserConfigurationModel().showTrace = showTrace;
                notifyConsumers('showTrace', getUserConfigurationModel().showTrace);
            }
            return getUserConfigurationModel().showTrace;
        };

        /**
         * [timeZone description]
         * @param  {[type]} tz [description]
         * @return {[type]}    [description]
         */
        self.timeZone = function(tz) {
            if (tz != undefined) {
                // set the timezone
                getUserConfigurationModel().timeZone = tz;

                //getUserConfigurationModel().save();
            }
            return getUserConfigurationModel().timeZone;
        };

         self.layoutFit = function(fit) {
            if (fit != undefined) {
                // set the timezone
                getUserConfigurationModel().layoutFit = fit;

                //getUserConfigurationModel().save();
            }
            return getUserConfigurationModel().layoutFit;
        };

        self.interactiveKiosk = function(kioskSettings) {
            if (kioskSettings != undefined) {
                // set the kiosk settings
                getUserConfigurationModel().modules.interactiveKiosk = kioskSettings;
            }
            return getUserConfigurationModel().modules.interactiveKiosk;
        };

        self.useNTP = function(useNTP) {
            if (useNTP != undefined) {
                // set the timezone
                getUserConfigurationModel().useNTP = useNTP;

                //getUserConfigurationModel().save();
            }
            return getUserConfigurationModel().useNTP;
        };

        self.usePMModeForScreenManagement = function(usePMModeForScreenManagement) {
            if (usePMModeForScreenManagement != undefined) {
                // set the PM Mode
                getUserConfigurationModel().usePMModeForScreenManagement = usePMModeForScreenManagement;
            }
            return getUserConfigurationModel().usePMModeForScreenManagement;
        };

        /**
         * [interativeKiosk description]
         * @param  {[type]} tz [description]
         * @return {[type]}    [description]
         */
        self.modules = function(modules) {
            if (modules != undefined) {
                // set the timezone
                getUserConfigurationModel().modules = modules;

                //getUserConfigurationModel().save();
            }

            return getUserConfigurationModel().modules;
        };

        /**
         * [remoteFirmwareUpdates description]
         * @param  {[type]} tz [description]
         * @return {[type]}    [description]
         */
        self.remoteFirmwareUpdates = function(remoteFirmwareUpdates) {
            if (remoteFirmwareUpdates != undefined) {
                // set the timezone
                getUserConfigurationModel().remoteFirmwareUpdates = remoteFirmwareUpdates;

                //getUserConfigurationModel().save();
            }

            return getUserConfigurationModel().remoteFirmwareUpdates;
        };

        /**
         * [getUserConfigurationModel description]
         * @return {[type]} [description]
         */
        function getUserConfigurationModel() {

            return _userConfiguration;
        }

        self.saveSettings = function() {
            return getUserConfigurationModel().save();
        };

        /**
         * [notifyConsumers description]
         * @param  {[type]} event [description]
         * @param  {[type]} data  [description]
         * @return {[type]}       [description]
         */
        function notifyConsumers(event, data) {

            //getUserConfigurationModel().save();

            $(_propertyChangedHandlers).each(function(i, handler) {
                handler(event, data);
            });
        }

    }

    return new UserConfigurationController();
});