String.prototype.startsWith||Object.defineProperty(String.prototype,"startsWith",{value:function(t,r){var e=r>0?0|r:0;return this.substring(e,e+t.length)===t}});

define(["system/io/File", "properties/App.Settings", "properties/App.Properties", "system/SystemInterface", "system/graphics/GraphicsInterface","system/network/NetworkInterface", "system/io/DirectoryInfo", "models/ConfigurationModel"], function(File, Settings, Properties, SystemInterface, GraphicsInterface, NetworkInterface, DirectoryInfo, ConfigurationModel) {
    function WidgetEventController() {
        var self = this;
        self.viewers = [];

        var readyEventAdditionalWaitInMS = 250; // wait half a second before showing widget
        var readyEventTimeout = null;

        // Setup event handler
        self.initialize = function() {
            logger.info("Widget event controller initialized");
            window.removeEventListener("message", widgetEventHandler);
            window.addEventListener("message", widgetEventHandler);
        };

        self.stop = function() {
            clearTimeout(readyEventTimeout);
            self.viewers = [];
        };

        function widgetEventHandler(event) {
            logger.debug('Widget SDK event received');
            if (event && event.data) {
                // Parse data
                var data = {};
                try {
                    if (typeof event.data === "string") {
                        data = JSON.parse(event.data);
                    }
                    else {
                        data = event.data;
                    } 
                } catch (error) {
                    logger.error('Unable to parse event data');
                    return;
                }
                // AgentM
                if (data.name && data.name.includes("Agent")) {
                    return;
                }
                // Check we have a method and if so continue to process request and response
                if (data.method) {
                    logger.info('Widget SDK event: ' + data.method);
                    if (data.params && data.params.constructor === Array) {
                        data.params.unshift(data.windowId); // If array then push windowId to front so it is the first param passed to all methods
                    } else {
                        data.params = [data.windowId]; // If no params are passed then the windowId is our only param
                    }
                    var childIframe = document.getElementById(data.windowId);
                    //var methodToCall = self[data.method];
                    var methodToCall = data.method;
                    // If the player is using an older version of the SDK then it is possible they will just send the callback rather than the success and error callbacks
                    var successCallback = data.successCallback
                        ? data.successCallback
                        : data.callback;
                    if (methodToCall) {
                        processRequestAndResponse(childIframe, methodToCall, data.params, data.messageId, successCallback, data.errorCallback);
                    } else {
                        logger.error('Requested method not supported');
                    }
                } else {
                    // Ignore as this is likely not related to the SL Widget SDK - could be a meldCX postMessage or something else
                    //logger.error('Invalid event data');
                }
            } else {
                logger.error('Invalid event data');
            }
        }

        function processRequestAndResponse(iframe, methodToCall, params, messageId, successCallback, errorCallback) {
            // This code originally used Function.apply() to dynamically pass the params to the method to call.
            // This function is unsupported on webOS 2 so this code was modified to call the implemented methods
            // using a switch statement instead.
            var requestProcessed = null;
            switch (methodToCall)
            {
                case "ready":
                    var windowId = params[0];
                    requestProcessed = self.ready(windowId);
                    break;
                case "getData":
                    var windowId = params[0];
                    var key = params[1];
                    var shared = params[2];
                    requestProcessed = self.getData(windowId, key, shared);
                    break;
                case "setData":
                    var windowId = params[0];
                    var key = params[1];
                    var value = params[2];
                    var shared = params[3];
                    requestProcessed = self.setData(windowId, key, value, shared);
                    break;
                case "getPlayerDetails":
                    var windowId = params[0];
                    requestProcessed = self.getPlayerDetails(windowId);
                    break;
                case "getDisplayProperties":
                    var windowId = params[0];
                    requestProcessed = self.getDisplayProperties(windowId);
                    break;
                case "getTLSSupportInfo":
                    var windowId = params[0];
                    requestProcessed = self.getTLSSupportInfo(windowId);
                    break;
                case "checkSupportedHTML5Features":
                    var windowId = params[0];
                    var features = params[1];
                    requestProcessed = self.checkSupportedHTML5Features(windowId, features);
                    break;
                case "requestMediaPlayerMoveToNextAsset":
                    var windowId = params[0];
                    requestProcessed = self.requestMediaPlayerMoveToNextAsset(windowId);
                    break;
                case "getOnlineStatus":
                    var windowId = params[0];
                    requestProcessed = self.getOnlineStatus(windowId);
                    break;
                case "listStoredDataKeys":
                    var windowId = params[0];
                    var shared = params[1];
                    requestProcessed = self.listStoredDataKeys(windowId, shared);
                    break;
                case "removeData":
                    var windowId = params[0];
                    var key = params[1];
                    var shared = params[2];
                    requestProcessed = self.removeData(windowId, key, shared);
                    break;
                case "clearStorage":
                    var windowId = params[0];
                    requestProcessed = self.clearStorage(windowId);
                    break;
                case "getExternalData":
                    var windowId = params[0];
                    var dataSource = params[1];
                    var integrationId = params[2];
                    var integrationName = params[3];
                    requestProcessed = self.getExternalData(windowId, dataSource, integrationId, integrationName);
                    break;
                default:
                    // Default response - Unsupported
                    requestProcessed = $.Deferred.reject({"reason": "Unsupported method call"});
                    break;
            };
            // Each method returns a deferred object. Once done we post the result back to the content Window
            requestProcessed.done(function(result) {
                if (successCallback) {
                    var responseMessage = {
                        messageId: messageId,
                        method: successCallback,
                        returnValue: result
                    }
                    if (iframe) {
                        iframe.contentWindow.postMessage(JSON.stringify(responseMessage), '*');
                    }
                }
            });
            requestProcessed.fail(function(error) {
                // If using the original version of the SDK there will not be an error callback so just
                // use the success callbak returning the error object
                var callbackToUse = errorCallback
                    ? errorCallback
                    : successCallback;
                if (callbackToUse) {
                    var responseMessage = {
                        messageId: messageId,
                        method: callbackToUse,
                        returnValue: error
                    }
                    if (iframe) {
                        iframe.contentWindow.postMessage(JSON.stringify(responseMessage), '*');
                    }
                }
            });
        }

        self.registerWidgetViewer = function(viewer) {
            var existingViewers = self.viewers.filter(function(existingViewer) {
                return existingViewer.id === viewer.id;
            });

            if (existingViewers.length === 0) {
                self.viewers.push(viewer);
            }
        };

        self.notifyWidgetsOfExternalDataUpdate = function(mediaAsset, integration, data) {
            var windowsThatNeedToBeNotified = getWindowsByMediaAssetId(mediaAsset.externalId);
            logger.info('Number of Windows to update: ' + windowsThatNeedToBeNotified.length);
            $.each(windowsThatNeedToBeNotified, function(index, window) {
                var responseData = {
                    dataSource: integration.dataSource,
                    integrationId: integration.integrationId,
                    integrationName: integration.name,
                    data: data
                };
                sendEventMessage(window.$activeViewer.attr('id'), "onExternalDataUpdate", responseData);
            });
        };

        // IMPLEMENTED EVENTS
        // ------------------------

        // Get data using file store - defaults to a namespace unique to zone and widget but can be stored globally if global is true
        self.getData = function(windowId, key, shared) {
            var window = getWindowById(windowId)
            if (window == null) {
                logger.warning('Widget attempted to get data but no matching viewer found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
            // Key to get data. If global is false then this will use isolated storage for this zone and asset.
            // If global is true then it will just store the key as is risking collissions with other widget data
            var fullKey = 'widget_data_' + key;
            if (shared !== true) { // false or undefined
                fullKey += '_' + windowId.substring(0, windowId.length - 2);
            }
            // Figure out path to file
            var widgetBasePath = '';
            if (Settings.application.fileSystem.root !== "") {
                widgetBasePath += Settings.application.fileSystem.root + "/";
            }
            if (Settings.application.fileSystem.widgetDirectory !== "") {
                widgetBasePath += Settings.application.fileSystem.widgetDirectory + '/';
            }
            widgetBasePath += window.mediaAsset.localFileName.replace('.wgt', '') + '/';
            var path = widgetBasePath + fullKey + '.json';
            var dfd = $.Deferred();
            File.read(path)
                .done(function(data) {
                    logger.info('Returning widget data');
                    dfd.resolve(data);
                })
                .fail(function() {
                    logger.error('Error getting widget data');
                    dfd.resolve(null);
                });
            return dfd;
        };

        // Store data in file store - defaults to a namespace unique to zone and widget but can be stored globally if global is true
        self.setData = function(windowId, key, value, shared) {
            var window = getWindowById(windowId)
            if (window == null) {
                logger.warning('Widget attempted to store data but no matching viewer found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
            // Key to get data. If global is false then this will use isolated storage for this zone and asset.
            // If global is true then it will just store the key as is risking collissions with other widget data
            var fullKey = 'widget_data_' + key;
            if (shared !== true) { // false or undefined
                fullKey += '_' + windowId.substring(0, windowId.length - 2);
            }
            // Figure out path to file
            var widgetBasePath = '';
            if (Settings.application.fileSystem.root !== "") {
                widgetBasePath += Settings.application.fileSystem.root + "/";
            }
            if (Settings.application.fileSystem.widgetDirectory !== "") {
                widgetBasePath += Settings.application.fileSystem.widgetDirectory + '/';
            }
            widgetBasePath += window.mediaAsset.localFileName.replace('.wgt', '') + '/';
            var path = widgetBasePath + fullKey + '.json';
            var dfd = $.Deferred();
            File.write(path, value)
                .done(function() {
                    logger.info('Widget data stored in file');
                    dfd.resolve(true);
                })
                .fail(function(error) {
                    logger.error('Error writing widget data to storage: ' + error);
                    dfd.resolve(false)
                });
            return dfd;
        };

        self.listStoredDataKeys = function(windowId, shared) {
            var window = getWindowById(windowId)
            if (window == null) {
                logger.warning('Widget attempted to store data but no matching viewer found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
            // Figure out path to file
            var dfd = $.Deferred();
            var widgetBasePath = '';
            if (Settings.application.fileSystem.root !== "") {
                widgetBasePath += Settings.application.fileSystem.root + "/";
            }
            if (Settings.application.fileSystem.widgetDirectory !== "") {
                widgetBasePath += Settings.application.fileSystem.widgetDirectory + '/';
            }
            widgetBasePath += window.mediaAsset.localFileName.replace('.wgt', '') + '/';

            var widgetDirInfo = new DirectoryInfo(widgetBasePath);
            widgetDirInfo.initialize().always(function() {
                var matchingKeysToReturn = [];
                $(widgetDirInfo.files).each(function(i, file) {
                    var key = file.name.replace('widget_data_', '').replace('.json', '').replace('_' + windowId.substring(0, windowId.length - 2), '');
                    if (file.name.startsWith('widget_data_')) {

                        if (shared === true) {
                            matchingKeysToReturn.push(key);
                        }
                        else {
                            if (file.name.includes(windowId.substring(0, windowId.length - 2))) {
                                matchingKeysToReturn.push(key);
                            }
                        }
                    }
                });
                dfd.resolve(matchingKeysToReturn);
            });
            return dfd;
        };

        self.removeData = function(windowId, key, shared) {
            var window = getWindowById(windowId)
            if (window == null) {
                logger.warning('Widget attempted to remove data but no matching viewer found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
            // Key to get data. If global is false then this will use isolated storage for this zone and asset.
            // If global is true then it will just store the key as is risking collissions with other widget data
            var fullKey = 'widget_data_' + key;
            if (shared !== true) { // false or undefined
                fullKey += '_' + windowIdwindowId.substring(0, windowId.length - 2);
            }
            // Figure out path to file
            var widgetBasePath = '';
            if (Settings.application.fileSystem.root !== "") {
                widgetBasePath += Settings.application.fileSystem.root + "/";
            }
            if (Settings.application.fileSystem.widgetDirectory !== "") {
                widgetBasePath += Settings.application.fileSystem.widgetDirectory + '/';
            }
            widgetBasePath += window.mediaAsset.localFileName.replace('.wgt', '') + '/';
            var path = widgetBasePath + fullKey + '.json';
            var dfd = $.Deferred();
            File.delete(path)
                .done(function() {
                    logger.info('Successfully deleted widget data');
                    dfd.resolve(true);
                })
                .fail(function() {
                    logger.error('Error removing widget data');
                    dfd.resolve(false);
                });
            return dfd;
        };

        self.clearStorage = function(windowId) {
            var window = getWindowById(windowId)
            if (window == null) {
                logger.warning('Widget attempted to store data but no matching viewer found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
            var dfd = $.Deferred();
            // Figure out path to file
            var widgetBasePath = '';
            if (Settings.application.fileSystem.root !== "") {
                widgetBasePath += Settings.application.fileSystem.root + "/";
            }
            if (Settings.application.fileSystem.widgetDirectory !== "") {
                widgetBasePath += Settings.application.fileSystem.widgetDirectory + '/';
            }
            widgetBasePath += window.mediaAsset.localFileName.replace('.wgt', '') + '/';

            var widgetDirInfo = new DirectoryInfo(widgetBasePath);
            widgetDirInfo.initialize().always(function() {
                var filesToDelete = [];
                $(widgetDirInfo.files).each(function(i, file) {
                    if (file.name.startsWith('widget_data_')) {
                        filesToDelete.push(file);
                    }
                });
                
                // delete each file synchronously
                var deleteDfd = $.Deferred().resolve();
                filesToDelete.forEach(function(fileToDelete) {
                    deleteDfd = deleteDfd.then(function() {
                        return fileToDelete.delete();
                    });
                });

                // report that deletion process has completed
                deleteDfd.done(function() {
                    logger.debug('Old files deleted.');
                    dfd.resolve();
                });
            });
            return dfd;
        };

        // Will 
        self.ready = function(windowId) {
            var dfd = $.Deferred();
            var window = getWindowById(windowId)
            if (window) {
                logger.debug('Widget ready event recieved. Showing widget..');
                readyEventTimeout = setTimeout(function() {
                    window.show();
                }, readyEventAdditionalWaitInMS);
                dfd.resolve();
            }
            else {
                logger.warning('Matching viewer not found found');
                dfd.reject({"reason": "Viewer not found"});
            }
            return dfd;
        };

        self.sendWidgetClosingNotification = function(windowId) {
            sendEventMessage(windowId, 'onWidgetClosingNotificationReceived', null);
        }

        function sendEventMessage(windowId, eventName, data) {
            var iframe = document.getElementById(windowId);
            var responseMessage = {
                messageId: 'videoEvent',
                method: eventName,
                returnValue: data
            }
            if (iframe) {
                iframe.contentWindow.postMessage(JSON.stringify(responseMessage), '*');
            }
        }

        self.getExternalData = function(windowId, dataSource, integrationId, objectId) {
            // dataSource is always DSS right now so can be ignored
            var window = getWindowById(windowId);
            if (window) {
                // Data dir
                var dataFilePath = '';
                if (Settings.application.fileSystem.root !== "") {
                    dataFilePath += Settings.application.fileSystem.root + "/";
                }
                if (Settings.application.fileSystem.widgetDirectory !== "") {
                    dataFilePath += Settings.application.fileSystem.widgetDirectory + '/';
                }
                dataFilePath += window.mediaAsset.localFileName.replace('.wgt', '') + "/";
                dataFilePath += Settings.application.fileSystem.integrationsWidgetSubdirectory + "/";
                dataFilePath += objectId + "/data.json";

                var dfd = $.Deferred();
                File.read(dataFilePath)
                    .done(function(data) {
                        try {
                            var deserializedData = JSON.parse(data);
                            dfd.resolve(deserializedData);
                        }
                        catch(error) {
                            logger.error(error);
                            dfd.reject({"reason": error});
                        }
                    })
                    .fail(function(error) {
                        logger.error(error);
                        dfd.reject({"reason": error});
                    });
                return dfd;

            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        self.getPlayerDetails = function(windowId) {
            var window = getWindowById(windowId);
            if (window) {
                return self._getPlayerDetails();
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        }

        self._getPlayerDetails = function() {
            var dfd = $.Deferred();
            var details = {};
			try {
                // Read Config file
				File.read(Settings.application.fileSystem.systemFiles.lastConfigFilePath).done(function(data) {
					// Load Config
					var configuration = new ConfigurationModel(JSON.parse(data));
					// Populate configuration data
					if (configuration && configuration.clientDetails)	
					{
                        var clientDetails = configuration.clientDetails;
                        details.configuration_change_id = configuration.changeId;
                        details.serial_number = clientDetails.signageliveSerialNumber;
                        details.site_description = clientDetails.siteDescription;
                        details.address_1 = clientDetails.address1;
                        details.address_2 = clientDetails.address2;
                        details.city = clientDetails.city;
                        details.state = clientDetails.state;
                        details.zip_code = clientDetails.zipCode;
                        details.reference_code_1 = clientDetails.referenceCode1;
                        details.reference_code_2 = clientDetails.referenceCode2;
                        details.reference_code_3 = clientDetails.referenceCode3;
                        details.reference_code_4 = clientDetails.referenceCode4;
                        details.reference_code_5 = clientDetails.referenceCode5;
                        details.reference_code_6 = clientDetails.referenceCode6;
                        details.reference_code_7 = clientDetails.referenceCode7;
                        details.reference_code_8 = clientDetails.referenceCode8;
                        details.reference_code_9 = clientDetails.referenceCode9;
                        details.reference_code_10 = clientDetails.referenceCode10;
                        details.time_zone = clientDetails.timeZone;
                        details.utc_offset = clientDetails.utcOffset;
                        details.client_version = Properties.appVersion;
                        details.client_build_number = Properties.appBuildNumber;
                        details.player_type = SystemInterface.getPlayerType();
                        SystemInterface.getModel().always(function(model) {
                            details.model = model;
                            SystemInterface.getManufacturer().always(function(manufacturer) {
                                details.manufacturer = manufacturer;
                                SystemInterface.getOSVersion().always(function(osVersion){
                                    details.os_version = osVersion;
                                    details.firmware_version = SystemInterface.getFirmwareVersionSync();
                                    dfd.resolve(details);
                                });
                            });
                        });
					}
				})
				.fail(function() {
					logger.error("Error loading device properties. Unable to read config file.");
                    dfd.reject({"reason": "Unable to read player configuration file"});
				});
			} catch(e) {
				logger.error('Error loading device properties: ' + e.message);
				dfd.reject({"reason": "Error loading device properties"});
			}
            return dfd;
        };

        self.getDisplayProperties = function(windowId) {
            var window = getWindowById(windowId);
            if (window) {
                return GraphicsInterface.getDisplayProperties();
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        self.getTLSSupportInfo = function(windowId) {
            var window = getWindowById(windowId);
            if (window) {
                return SystemInterface.getMaxSupportedTLSVersion();
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        self.checkSupportedHTML5Features = function(windowId, features) {
            var window = getWindowById(windowId);
            if (window) {
                return SystemInterface.checkSupportedHTML5Features(features);
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        self.requestMediaPlayerMoveToNextAsset = function(windowId) {
            var window = getWindowById(windowId);
            if (window) {
                return window.requestMediaPlayerMoveToNextAsset();
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        self.getOnlineStatus = function(windowId) {
            var window = getWindowById(windowId);
            if (window) {
                return NetworkInterface.getOnlineStatus();
            }
            else {
                logger.warning('Matching viewer not found found');
                return $.Deferred().reject({"reason": "Viewer not found"});
            }
        };

        function getWindowById(windowId) {
            var viewerId = windowId.substring(0, windowId.length - 2);  
            var existingViewers = self.viewers.filter(function(existingViewer) {
                return existingViewer.id === viewerId;
            });
            var window = (existingViewers.length === 1)
                ? existingViewers[0]
                : null;
            return window;
        }

        function getWindowsByMediaAssetId(maId) {
            return self.viewers.filter(function(viewer) {
                return viewer.mediaAsset && viewer.mediaAsset.externalId === maId;
            });
        }
    }

    return new WidgetEventController();
});