define(["properties/App.Settings", "system/DateTime", "system/io/File"], function(Settings, DateTime, File) {   
    function LogUploadInterfaceHelper() {
        var self = this;

        self.uploadLogFile = function(s3KeyName, logData) {
            var dfd = $.Deferred();
            logger.info('Uploading log file: ' + s3KeyName);
            try {
                // Configure request body for request to get upload url
                var bodyData = {
                    key: s3KeyName,
                    contentType: 'text/plain'
                };
                // Get the upload url
                getUploadUrlResponse(Settings.application.modules.logging.requestUploadUrlEndpoint, bodyData).then(function(res) {
                    var uploadURL = res.uploadURL;

                    if (Settings.application.modules.logging.useSLDomain === true) {
                        uploadURL = uploadURL.replace(Settings.application.modules.logging.AWSUploadDomain, Settings.application.modules.logging.SLUploadDomain);
                    }

                    // Do the log upload 
                    sendLog(uploadURL).then(function() {
                        dfd.resolve();
                    });
                });
            }
            catch (err) {
                logger.error('An Error occurred attempting to upload logs. Message: ' + err.message);
                return dfd.resolve();
            }
            return dfd;
        }

        function getUploadUrlResponse(requestUrlEndpoint, bodyData) {
            var dfd = $.Deferred();
            try {
                return $.ajax({
                    url: requestUrlEndpoint,
                    data: JSON.stringify(bodyData),
                    type: 'PUT',
                    contentType: 'application/json',
                    timeout: 60000
                });
            } catch (e) {
				logger.error(e);
                dfd.reject(e);
            }
            return dfd;
        };

        function sendLog(url, body) {
            var dfd = $.Deferred();
            try {
                return $.ajax({
                    url: url,
                    data: body,
                    type: 'PUT',
                    contentType: 'text/plain',
                    timeout: 60000
                });
            } catch (e) {
				logger.error(e);
                dfd.reject(e);
            }
            return dfd;
        };

    }
    return new LogUploadInterfaceHelper();
});