define(["properties/App.Settings","properties/App.Properties", "models/API", "system/DateTime","system/io/File"], function(Settings, Properties, API, DateTime, File) {
    function SLAPIMethodsInterface() {
    
        var self = this;

        function getEndpoint(api, useLegacyTlsUrl) {
            var apiEndpoint = "";
            switch(api) {
                case API.PLAYER_API:
                    apiEndpoint = useLegacyTlsUrl
                        ? Settings.application.api.legacyTlsUrl
                        : Settings.application.api.url;
                    break;
                case API.SCREENSHOT_UPLOAD:
                    apiEndpoint = useLegacyTlsUrl
                        ? Settings.application.screenshotuploadapi.legacyTlsUrl
                        : Settings.application.screenshotuploadapi.url;
                    break;
                case API.PLAYER_DATA:
                    apiEndpoint = useLegacyTlsUrl
                        ? Settings.application.playerdataapi.legacyTlsUrl
                        : Settings.application.playerdataapi.url;
                    break;
                case API.WEB_TRIGGERS:
                    apiEndpoint = useLegacyTlsUrl
                        ? Settings.application.webBasedInterrupts.legacyTlsUrl
                        : Settings.application.webBasedInterrupts.url;
                    break;
            }
            return apiEndpoint;
        }

        self.deviceSupportCheckAPICallFailed = function() {
            if (Settings.application.features.supportsTLS12 === false) {
                Settings.application.api.useLegacyTlsUrl = !Settings.application.api.useLegacyTlsUrl;
            }
        };
        
        self.registerAPICallFailed = function() {
            if (Settings.application.features.supportsTLS12 === false) {
                Settings.application.api.useLegacyTlsUrl = !Settings.application.api.useLegacyTlsUrl;
            }
        };

        self.authenticate = function(api, useDefaultUrl) {
            var dfd = $.Deferred();
            // Always default to using legacy API for authentication if device does not support TLS 1.2
            if (api == API.PLAYER_API && Settings.application.features.supportsTLS12 === false) {
                Settings.application.api.useLegacyTlsUrl = true;
            }
            logger.info('Auth: Use Default Url: ' + useDefaultUrl);
            var jqXHR = self.executeAPICall(api,'authenticate', {
                type: 'POST'
            }, useDefaultUrl);

            jqXHR.done(function(data, textStatus, jqXHR) {
                logger.debug('authenticated');
                // set the auth token
                Settings.application.api.authToken = data.token;
                Settings.application.api.authTokenExpiry = DateTime.now().addMinutes(30);

                // This code is executed for Chrome players after they have successfully authenticated with the SL API
                // for the first time after switching to using the Device Directory Id as their client Id.
                if (Settings.application.api.temp && Settings.application.api.temp.deleteClientIdFileOnAuthenticationSuccess === true) {
                    File.delete(Settings.application.fileSystem.systemFiles.clientIdFilePath)
                        .done(function() {
                            logger.info('Deleted clientId File as it is no longer required. Successfully transitioned to using the Device Directory Id');
                        })
                        .fail(function() {
                            logger.error('Unable to delete clientId file. Unable to successfully transition to Device Directory Id.');
                        });
                }
                // We are authenticated
                dfd.resolve(true);
            }).fail(function(jqXHR, textStatus, errorThrown) {
                logger.warning('Auth failed');
                // log the error
                logAPIError(this.url, this.data, jqXHR.status, errorThrown, jqXHR.responseText);
                // If webOS 1 and we get a TLS error, we will default back to the original URL
                if (api == API.PLAYER_API && Settings.application.features.supportsTLS12 === false && jqXHR.status != 401 && !useDefaultUrl) {
                    logger.info('Attempting to reauthenticate using default url...');
                    self.authenticate(api,true).done(function(result){
                        if(result) {
                            Settings.application.api.useLegacyTlsUrl = false;
                            logger.info('Was unable to connect to legacy TLS URL but could authenticate with regular URL.');
                        } else {
                            // Some other error, default back to TLS 1 URL
                            logger.info('Authenticated using legacy TLS URL (NENA).');
                            Settings.application.api.useLegacyTlsUrl = true;
                        }
                        dfd.resolve(result);
                    });
                }
                else {
                    // auth failed   
                    // attempt to re-athenticate in a few seconds
                    //_authenticationTimerId = setTimeout(authenticate, _authenticationFailureRetryIntervalMS);
                    dfd.resolve(false);
                }
            });
            return dfd;
        };

        self.executeAPICall = function(api, operationName, settings, forceUseDefaultUrl) {
            try {
                var useLegacyTlsUrl = false;       
                // If we aren't forcing the default Url, select based on the OS version and API setting
                if (!forceUseDefaultUrl) {
                    useLegacyTlsUrl = Settings.application.features.supportsTLS12 === false
                        ? Settings.application.api.useLegacyTlsUrl
                        : false;
                }
                logger.info("Use legacy TLS url: " + useLegacyTlsUrl);
                var apiEndpoint = getEndpoint(api, useLegacyTlsUrl);
                logger.info(api + ': executeAPICall ' + operationName + ', URL: ' + apiEndpoint);
                // Request Defaults
                var type = 'GET';
                var async = true;
                var data = null;                        
                var contentType = 'application/json';
                var dataType = null;
                var timeout = 60000;

                // Apply passed settings
                if (settings != undefined) {
                    if (settings.type != undefined) {
                        type = settings.type;
                    }
                    if (settings.async != undefined) {
                        async = settings.async;
                    }
                    if (settings.data != undefined) {
                        if (api == API.PLAYER_DATA && operationName == 'proofofplay') {
                            data = settings.data;
                        } else {
                            data = JSON.stringify(settings.data);
                        }
                    }
                    if (settings.contentType != undefined) {
                        contentType = settings.contentType;
                    }
                    if (settings.dataType != undefined) {
                        dataType = settings.dataType;
                    }
                    if (settings.timeout != undefined) {
                        timeout = settings.timeout;
                    }
                }
                // If a dataType was not specifically defined and this is not a head request then default to JSON
                if ((settings == undefined || settings.dataType == undefined) && type != 'HEAD') {
                    dataType = 'json';
                }
                if (api == API.PLAYER_DATA && operationName == 'proofofplay') {
                    contentType = 'text/plain;charset=UTF-8;';
                    dataType = 'text';
                    timeout = 0;
                }
                if (api != API.WEB_TRIGGERS) {
                    logger.info(api + ': API Credentials - Client ID: ' + Settings.application.api.clientId + ', Token: ' + Settings.application.api.authToken + ", Expiry: " + Settings.application.api.authTokenExpiry);
                }
                // Do request
                return $.ajax({
                    url: apiEndpoint + "/" + operationName,
                    type: type,
                    contentType: contentType,
                    dataType: dataType,
                    async: async,
                    data: data,
                    timeout: timeout,
                    beforeSend: function(xhr, settings) {
                        if (api != API.WEB_TRIGGERS) {
                            xhr.setRequestHeader('X-SIGNAGELIVE-APP-ID', Settings.application.api.appId);
                            xhr.setRequestHeader('X-SIGNAGELIVE-APP-KEY', Settings.application.api.appKey);
                            xhr.setRequestHeader('X-SIGNAGELIVE-CLIENT-ID', Settings.application.api.clientId);
                            xhr.setRequestHeader('X-SIGNAGELIVE-CLIENT-KEY', Settings.application.api.clientKey);
                            xhr.setRequestHeader('X-SIGNAGELIVE-AUTH-TOKEN', Settings.application.api.authToken);
                            if (Settings.application.api.temp && Settings.application.api.temp.setOldClientIdHeader === true) {
                                xhr.setRequestHeader('X-SIGNAGELIVE-OLD-CLIENT-ID', Settings.application.api.temp.oldClientId);
                            }
                            if (Properties.appBranding) {
                                xhr.setRequestHeader('X-SIGNAGELIVE-INSTALL-ID', Properties.appBranding);
                            }
                            else {
                                if (Settings.application.api.brand !== 'Signagelive') {
                                    xhr.setRequestHeader('X-SIGNAGELIVE-INSTALL-ID', Settings.application.api.brand);
                                }
                            }
                        }
                    }
                });
            } catch (e) {
                logger.error(api + ': Error executing api call ' + e.message);
                return null;
            }
        };

       
        function logAPIError(requestUrl, requestData, responseStatus, responseMessage, responseText) {
            var logData = {
                url: requestUrl,
                requestData: requestData,
                responseStatus: responseStatus,
                responseMessage: responseMessage,
                responseBody: responseText
            };

            logger.error("PLAYER DATA API: API Error " + JSON.stringify(logData));
        }
    }

    return SLAPIMethodsInterface;
});