define(["properties/App.Settings"], function(Settings) {
    function VideoPlayer(parentId, x, y, width, height, zIndex, callbacks) {
        var self = this;
        var _callbacks = callbacks;
        var $videoViewer;
        var _stopped = false;
        self.parentId = parentId;
        self.zIndex = zIndex;
        self.id = parentId + "_videoViewer";

        var BH5CTransformTimeout = null;

        // Max res 1919x1079 not 1920x1080
        var videoWidth = width;
        var videoHeight = height;

        // Ensure Max constraints are applied to
        if (Settings.application.webOSVersion <= 2) {
            logger.info("WebOS version is 1.0 or 2.0 - restricting video size to 1919x1079.");
            if (videoWidth == 1920) videoWidth = 1919;
            if (videoHeight == 1080) videoHeight = 1079;
        }

        self.html = "<div id = '" + self.id + "_container' style='background-color:#000;visibility:hidden;position:absolute;left:" + x + "px;top:" + y + "px;width:" + width + "px;height:" + height + "px;z-index: " + self.zIndex + ";'><video id='" + self.id + "' style='visibility:hidden;position:absolute;left:0px;top:0px;z-index: " + self.zIndex + ";' width='" + videoWidth + "' height='" + videoHeight + "' autoplay></video></div>";
        // Only apply the texture attribute to webOS panels > 3.2
        self.videoHtml = "<video id='" + self.id + "' style='object-fit:fill;visibility:hidden;position:absolute;left:0px;top:0px;width:" + videoWidth + "px;height:" + videoHeight + "px;z-index: " + self.zIndex + "' autoplay></video>";
        
        self.IPTVHtml = "<video id='" + self.id + "' style='object-fit:fill;position:absolute;left:0px;top:0px;width:" + videoWidth + "px;height:" + videoHeight + "px;z-index: " + self.zIndex + "'></video>";

        var _playingGapless = false;
        var _playing = false;

        self.addedToCanvas = function() {
            $videoViewer = $('#' + self.id);
            $videoViewerContainer = $('#' + self.id + "_container");

            if (Settings.application.model === '86BH5C' && videoWidth === 3840 && videoHeight === 600) {
                logger.info('This is the 86BH5C panel playing a fullscreen video. Setting object-fit: contain')
                $videoViewer.css('object-fit', 'contain');
            }

            $videoViewer.on('playing', function() {
                logger.info("Video Playing event fired.");
                if (!_playingGapless && !_playing && _callbacks.playing != null && !_stopped) {
                    _playing = true;
                    _callbacks.playing();
                }
                self.show();
            });

            // NOTE We are using the pause event as this fires before the ended event
            // which seems to sometimes break
            $videoViewer.on('pause', function() {
                if (!_playingGapless && _playing) {
                    _playing = false;
                    logger.info("Video Pause event fired.");
                    //self.hide();
                    //$('#' + self.id).remove();
                    if (_callbacks.complete != null && !_stopped)
                        _callbacks.complete();
                }
            });

            $videoViewer.on('error', function() {
                logger.error("Video Error event fired.");
                _playing = false;
                self.hide();
                if (_callbacks.fail != null)
                    _callbacks.fail();
            });
        };

        self.play = function(url) {
            clearTimeout(BH5CTransformTimeout);
            _stopped = false;
            _playingGapless = false;
            _playing = false;
            // Everytime we play a video try removing and readding the video element entirely
            $('#' + self.id).remove();
            $('#' + self.id + "_container").append(self.videoHtml);
            self.addedToCanvas();

            // And set the video source
            $videoViewer.attr('src', url);
            
            if (Settings.application.model === '86BH5C') {
                transformVideoFor86BH5C();
            }
        };

        self.playGapless = function(src) {
            clearTimeout(BH5CTransformTimeout);
            _playingGapless = true;
            _playing = false;
            // Everytime we play a video try removing and readding the video element entirely
            $('#' + self.id).remove();
            $('#' + self.id + "_container").append(self.videoHtml);
            self.addedToCanvas();
            // Configure event handling
            var options = {};
            options.htmlMediaOption = {};
            options.htmlMediaOption.useUMSMediaInfo = true;
            // Create a source element - required?!
            $source = $("<source></source>");
            $source.attr('src', src);
            $source.attr('type', 'video/mp4;mediaOption=' + escape(JSON.stringify(options)));
            $videoViewer.append($source);
            // Custom gapless events
            $videoViewer.on('umsmediainfo', function(data) {
                logger.debug("Custom even handler called");
                var customEvent = data.originalEvent;
                var eventDetail = JSON.parse(customEvent.detail);
                // Handle Error(s)
                if (eventDetail.type == "error") {
                    _callbacks.gaplessFail(eventDetail.info);
                }
                // Handle individual video playback complete
                if (eventDetail.info.hasOwnProperty('gapless')) {
                    if (eventDetail.info.gapless.eventType == "EOF") {
                        logger.debug("EOF called");
                        _callbacks.gaplessComplete(eventDetail.info.gapless.mediaId);
                    }
                }
            });

            $videoViewer.on('error', function(e) {
                _playing = false;
                logger.debug("Gapless Video Error event fired.");
                self.hide();
                if (_callbacks.gaplessFail != null)
                    _callbacks.gaplessFail(e);
            });

            //$videoViewer.attr('src', src);
            $videoViewer[0].load();
            $videoViewer[0].play();
            
            if (Settings.application.model === '86BH5C') {
                transformVideoFor86BH5C();
            }
        };

        function transformVideoFor86BH5C() {
            logger.info('Transforming video for 86BH5C');
            var newX = x / 2;
            var newY = y / 2;
            var newWidth = videoWidth / 2;
            var newHeight = videoHeight / 2;
            logger.info('Updated settings: x: ' + newX + ', y: ' + newY + ', w: ' + newWidth + ', h: ' + newHeight);
            // Have to wait 2 seconds before transforming
            BH5CTransformTimeout = setTimeout(function() {
                new Video().setVideoViewTransform(
                    function() {
                        logger.info('Successfully transformed video for 86BH5C');
                    }, function(e) {
                        logger.error('Error transforming video 86BH5C: ' + e.errorCode + ': ' + e.errorText);
                    },
                    {
                        x: newX,
                        y: newY,
                        width: newWidth,
                        height: newHeight
                    }
                );
            }, 2000);
            
        }

        self.playIPTV = function(url) {
            clearTimeout(BH5CTransformTimeout);
            _playingGapless = false;
            // Everytime we play a video try removing and readding the video element entirely
            $('#' + self.id).remove();
            $('#' + self.id + "_container").append(self.IPTVHtml);
            $videoViewer = $('#' + self.id);

            $('#' + self.id).on('playing', function() {
                logger.debug("Video Playing event fired.");
                self.show();
            });

            $('#' + self.id).on('error', function() {
                logger.debug("Video Error event fired.");
                self.hide();
                if (_callbacks.fail != null)
                    _callbacks.fail();
            });

            // Figure out protocol
            var protocol = url.split("://")[0].toUpperCase();
            // Get video and create source eleemnt
            var video = document.getElementById(self.id);
            var source = document.createElement("source");
            // set src
            source.setAttribute("src", url);
            // webOS 2 supports mpegts only
            // webOS 3+ supports mp4 or mp2t but currently we have no easy way of differentiating the two
            // as Signagelive sends all streams with the mimeType of application/vnd.signagelive.iptv so for now just setting to mp4 for webOS 3.0+
            var mediaType = (Settings.application.webOSVersion > 2) ? 'mp4' : 'mpegts';
            logger.info('Media Type: ' + mediaType);
            // Configure protocol options - required for all streams except HLS (http/https)
            if (protocol === "UDP" || protocol === "RTP" || protocol === "RTSP") {
                var options = configureProtocolOptions(protocol);
                source.setAttribute("type", "video/" + mediaType + ";mediaOption=" + escape(JSON.stringify(options)));
            }
            // append source
            video.appendChild(source);
            // load and play
            video.load();
            video.play();

        };

        function configureProtocolOptions(protocol) {
            var options = {};
            options.htmlMediaOption = {};
            options.htmlMediaOption.useUMSMediaInfo = true;
            options.mediaTransportType = protocol.toUpperCase();
            return options;
        }

        self.stop = function() {
            logger.info("Stopping video playback");
            // unload the video
            _playingGapless = false;
            _playing = false;
            _stopped = true;
            self.hide();
            $videoViewer[0].pause();
        };

        self.removeEventListeners = function() {
            clearTimeout(BH5CTransformTimeout);
            if ($videoViewer) {
                $videoViewer.off('playing');
                $videoViewer.off('paused');
                $videoViewer.off('error');
            }
            _callbacks = {};
        }

        self.pause = function() {
            //
        };

        self.setMute = function(muted) {
            $videoViewer[0].muted = muted;
        };

        self.volumeUp = function() {};
        self.volumeDown = function() { };

        self.show = function() {
            $videoViewer.css('visibility', 'visible');
        };

        self.hide = function() {
            $videoViewer.css('visibility', 'hidden');
            $('#' + self.id).remove();
        };
    }

    return VideoPlayer;
});