define(["controllers/WidgetEventController", "properties/App.Settings"], function(WidgetEventController, Settings) {
    function WidgetViewer(parentId, x, y, width, height, callbacks) {
        var self = this;
        self.callbacks = callbacks;
        self.currentMAId = null;
        self.mediaAsset = null;
        self.id = parentId + '_widgetViewer';
        self.currentAssetDuration = null;
        var viewerOneId = parentId + '_widgetViewer_1';
        var viewerTwoId = parentId + '_widgetViewer_2';
        var html = "<iframe class='viewer' sandbox='allow-same-origin allow-scripts allow-forms' id='" + viewerOneId + "' name='" + viewerOneId + "' frameborder='0' scrolling='no' src='' style='position:absolute;left:0px;top:0px;width:" + width + "px;height:" + height + "px;visibility:hidden;'></iframe><iframe class='viewer' sandbox='allow-same-origin allow-scripts allow-forms' id='" + viewerTwoId + "' name='" + viewerTwoId + "' frameborder='0' scrolling='no' src='' style='position:absolute;left:0px;top:0px;width:" + width + "px;height:" + height + "px;visibility:hidden;'></iframe>";
        self.$activeViewer = null;
        self.$inactiveViewer = null;
        var showTimeout = null;
        var loadTimeout = null;
        var showing = false;
        var ignoreMoveNext = false;
        var defaultTimeToWaitForReadyEventInMS = 1500;

        self.appendToCanvas = function() {
            $('#' + parentId).append(html);
            WidgetEventController.registerWidgetViewer(self);
            self.$activeViewer = $('#' + viewerOneId);
            self.$inactiveViewer = $('#' + viewerTwoId);
        };

        self.play = function(url, duration, mediaAsset) {
            logger.info('Playing widget using widget viewer control. url to play: ' + url);
            ignoreMoveNext = false;
            showing = false;
            logger.info("Not Showing");
            self.mediaAsset = mediaAsset;
            self.currentAssetDuration = duration;
            self.$inactiveViewer.off('load').on('load', function() {
                self.$inactiveViewer.off('load');
                var currentSrc = self.$inactiveViewer.attr('src');                
                if (currentSrc !== 'about:blank') {
                    logger.info('Showing widget in 3 seconds if ready event is not received');
                    // Add texture attr to embedded video tags
                    if (Settings.application.webOSVersion === 3 || Settings.application.webOSVersion === 3.2) {
                        // logger.info('WebOS 3.0/3.2 panel - adding the texture attribute to all video elements within the widget');
                        // self.$inactiveViewer.contents().find("video").attr("texture", "");
                    }
                    // Play widget after 2.5 secs if ready event is not received
                    showTimeout = setTimeout(function() {
                        logger.info('Showing widget after 2.5 seconds - ready event was not recieved');
                        self.show();
                    }, defaultTimeToWaitForReadyEventInMS);
                }
            });
            self.$inactiveViewer.attr("src", url);
        };

        self.remove = function() {
            showing = false;
            logger.info("Not Showing");
            clearTimeout(showTimeout);
            clearTimeout(loadTimeout);
            showTimeout = null;
            self.callbacks = {};
            self.$activeViewer.remove();
            self.$inactiveViewer.remove();
        };

        self.show = function() {
            logger.info("Show Widget Viewer...");
            if (!showing && ignoreMoveNext === false) {
                showing = true;
                logger.info("Showing!");
                logger.info('Showing widget viewer control and clearing show timeout');
                clearTimeout(showTimeout);
                showTimeout = null;
                // Hide all other elements
                if (self.callbacks.ready) {
                    self.callbacks.ready(self.currentAssetDuration);
                }
                switchActiveViewer();
                self.$activeViewer.css('visibility', 'visible');
                loadTimeout = setTimeout(function() {
                    hideInactiveViewer();
                }, 250);
            }
            else {
                logger.warning('Attempted to show widget viewer control but it is already showing');
            }
        };

        self.hide = function() {
            logger.info('Hiding widget viewer control and clearing timeout');
            showing = false;
            logger.info("Not Showing");
            clearTimeout(showTimeout);
            clearTimeout(loadTimeout);
            showTimeout = null;
            hideActiveViewer();
        };

        self.stop = function() {
            ignoreMoveNext = false;
            self.sendWidgetClosingNotification();
            self.hide();
        };

        self.requestMediaPlayerMoveToNextAsset = function() {
            if (ignoreMoveNext === false) {
                ignoreMoveNext = true;
                clearTimeout(showTimeout);
                self.callbacks.moveNext();
            }
            return $.Deferred().resolve();
        }

        self.sendWidgetClosingNotification = function() {
            WidgetEventController.sendWidgetClosingNotification(self.$activeViewer.attr('id'));
        }

        function switchActiveViewer() {
            logger.debug('Switching active Viewer');
            var $temp = self.$activeViewer;
            self.$activeViewer = self.$inactiveViewer;
            self.$inactiveViewer = $temp;
        }

        function hideActiveViewer() {
            logger.debug('Hide active Viewer');
            self.$activeViewer = hideViewer(self.$activeViewer);
        }

        function hideInactiveViewer() {
            logger.debug('Hide inactive Viewer');
            self.$inactiveViewer = hideViewer(self.$inactiveViewer);
        }

        function hideViewer($viewer) {
            logger.debug('Hide viewer');
            try {
                var id = $viewer.attr('id');
                $("#" + id).css('visibility', 'hidden');
                var html = "<iframe class='viewer' sandbox='allow-same-origin allow-scripts allow-forms' id='" + id + "' name='" + id + "' frameborder='0' scrolling='no' src='' style='position:absolute;left:0px;top:0px;width:" + width + "px;height:" + height + "px;visibility:hidden;'></iframe>";
                logger.info('Replacing viewer id: ' + id);
                $("#" + id).remove();
                $('#' + parentId).append(html);
                var replacedViewer = $('#' + id);
                return replacedViewer;
            }
            catch (e) {
                logger.error(e);
            }
        }

        self.onKeyPress = function (key) {
            try {
                var widgetIframe = document.getElementById(self.id);
                if (widgetIframe) {
                    widgetIframe.contentWindow.onKeyPress(key);
                }
            } catch (err) {
                //Function not on widget
            }
        };

    }

    return WidgetViewer;
});