define(['lib/knockout', 'system/DateTime'], function(ko, DateTime) {
	function DateTimeViewModel() {
		var _monthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

		var now = DateTime.now();

		var _selectedDate = ko.observable(now.getDate());
		var _selectedMonth = ko.observable(now.getMonth()+1);
		var _selectedYear = ko.observable(now.getFullYear());
		var _selectedHour = ko.observable(now.getHours());
		var _selectedMinute = ko.observable(now.getMinutes());

		this.date = function() {
			// TODO calculate the difference between now and the last time the DT was adjusted in the UI
			// as due to the fact that the panel must reboot before the dt is saved we need to do this only once all other settings are completed.
			return new Date(_selectedYear(), _selectedMonth() - 1, _selectedDate(), _selectedHour(), _selectedMinute());
		}

		this.day = ko.computed(function() {
			return _selectedDate();
		}, this);

		this.month = ko.computed(function() {
			return _monthNames[_selectedMonth() - 1];
		}, this);

		this.year = ko.computed(function() {
			return _selectedYear();
		}, this);

		this.hour = ko.computed(function() {
			var padding = '00';
			var result = padding + _selectedHour().toString();
			return result.substr(-2, 2).toUpperCase();
		}, this);

		this.minute = ko.computed(function() {
			var padding = '00';
			var result = padding + _selectedMinute().toString();
			return result.substr(-2, 2).toUpperCase();
		}, this);

		this.addDays = function(n) {
			var newDate = _selectedDate() + n;

			var maxDateForMonth = new Date(_selectedYear(), _selectedMonth(), 0);
			if (newDate > maxDateForMonth.getDate()) {
				newDate = 1;
			}

			if (newDate == 0) {
				newDate = maxDateForMonth.getDate();
			}

			_selectedDate(newDate);
		}

		this.addMonths = function(n) {
			var newMonth = _selectedMonth() + n;

			if (newMonth == 13)
				newMonth = 1;

			if (newMonth == 0)
				newMonth = 12;

			var maxDateForMonth = new Date(_selectedYear(), newMonth, 0);
			if (_selectedDate() > maxDateForMonth.getDate()) {
				_selectedDate(maxDateForMonth.getDate());
			}

			_selectedMonth(newMonth);
		}

		this.addYears = function(n) {
			var newYear = _selectedYear() + n;

			var maxDateForMonth = new Date(newYear, _selectedMonth(), 0);
			if (_selectedDate() > maxDateForMonth.getDate()) {
				_selectedDate(maxDateForMonth.getDate());
			}

			_selectedYear(newYear);
		}

		this.addHours = function(n) {
			var newHour = _selectedHour() + n;

			if (newHour == 24)
				newHour = 0;

			if (newHour == -1)
				newHour = 23;

			_selectedHour(newHour);
		}

		this.addMinutes = function(n) {
			var newMinutes = _selectedMinute() + n;

			if (newMinutes == 60)
				newMinutes = 0;

			if (newMinutes == -1)
				newMinutes = 59;

			_selectedMinute(newMinutes);
		}
	}

	return DateTimeViewModel;
})
