define(["properties/App.Settings", "system/io/File", 'system/network/NetworkInterface', 'system/SystemInterface', 'properties/App.Properties', 'models/ConfigurationModel'], function(Settings, File, NetworkInterface, SystemInterface, Properties, ConfigurationModel) {
    /*
    	DevicePropertiesViewModel - Used by the DevicePropertiesView
     */
    function DevicePropertiesViewModel() {
        var self = this;

        // Defaults
        self.signageliveSerialNumber = '';
        self.changeId = '';
        self.macAddress = '';
        self.ipv4_address = 'Unknown';
        self.firmwareVersion = 'Unknown';
        self.modelNumber = 'Unknown';
        self.packageType = Settings.application.packageType;
        self.webOSVersion = Settings.application.webOSVersion;
        self.sdkVersion = Settings.application.sdkVersion

        //
        // Generates the model
        //
        self.generate = function() {

            var dfd = $.Deferred();

            try {

                // LRead Config file (async)
                File.read(Settings.application.fileSystem.systemFiles.lastConfigFilePath).done(function(data) {
                        // Load Config
                        var configuration = new ConfigurationModel(JSON.parse(data));
                        // Use Configuration data to get serialNo and changeId
                        self.signageliveSerialNumber = "";
                        self.changeId = "0";
                        if (configuration != null) {
                            self.changeId = configuration.changeId;
                            if (configuration.clientDetails != null && configuration.clientDetails.signageliveSerialNumber != null) {
                                self.signageliveSerialNumber = configuration.clientDetails.signageliveSerialNumber;
                            }
                        }
                        // Use Properties file to get app version and build number
                        self.clientVersion = Properties.appVersion;
                        self.buildNumber = Properties.appBuildNumber;
                        // Use NetworkInterface to get the Device Mac Address and IPAddress
                        NetworkInterface.getMAC().done(function(mac) {
                            self.macAddress = mac;
                        }).always(function() {
                            NetworkInterface.getIP().done(function(ip) {
                                self.ipv4_address = ip;
                            }).always(function() {
                                SystemInterface.getFirmwareVersion().done(function(firmwareVersion) {
                                    self.firmwareVersion = firmwareVersion;
                                }).always(function() {
                                    SystemInterface.getModel().done(function(modelNumber) {
                                        self.modelNumber = modelNumber;
                                        // Resolve model
                                        dfd.resolve();
                                    });
                                });
                            });
                        });

                    })
                    .fail(function() {
                        logger.error("Device Properties View Model: Error: Unable to read last configuration.");
                    });
            } catch (e) {
                logger.error('Device Properties View Model: Error loading the configuration data: ' + e.message);
                dfd.reject();
            }

            return dfd;
        };

    }

    return DevicePropertiesViewModel;
});