define(["gui/viewmodels/ZoneViewModel", "controllers/UserConfigurationController"], function(ZoneViewModel, UserConfigurationController) {
	function LayoutViewModel(layoutModel, screenWidth, screenHeight) {
		var self = this;

		// By default layouts are resized to fit the available display area by determining the scalar difference between the widget and height of the layout and the screen width and height
		var layoutFit = 'fit';
		var _xScalar = screenWidth / layoutModel.width;
		var _yScalar = screenHeight / layoutModel.height;

		// The layoutFit can be changed in the user preferences so that layouts are not resized. Load user-configured layoutFit
		// As this is a core module, protect against layoutFit() being undefined in cases where a media player may override the UserConfigurationController and not implement this

		if (typeof UserConfigurationController.layoutFit === "function") { 
			// safe to use the function
			if (UserConfigurationController.layoutFit() != null) {
				layoutFit = UserConfigurationController.layoutFit().id;
				logger.info('Layout Fit: ' + layoutFit);
			}
		}

		// Update scalar based on user configured layout fit
		switch (layoutFit) {
			case "fit":
				// Nothing to do as this is the default
				break;
			case "none":
				// Reset scalars back to 1 so the layout and the zones are not resized
				_xScalar = 1;
				_yScalar = 1;
				break;
			default:
				// Nothing to do
				break;
		}

		// Update layout width and height based on scalar
		self.width = layoutModel.width * _xScalar;
		self.height = layoutModel.height * _yScalar;
		// Pass scalar to the zone models so they can also be scalred appropriately
		self.zones = [];
		$(layoutModel.zones).each(function(i, zone) {
			self.zones.push(new ZoneViewModel(zone, _xScalar, _yScalar));
		});

		self.zones.sort(function(a, b) {
			// a and b will here be two objects from the array
		    // thus a[1] and b[1] will equal the names
		
		    // if they are equal, return 0 (no sorting)
		    if (a.zIndex == b.zIndex) { return 0; }
		    if (a.zIndex > b.zIndex)
		    {
		        // if a should come after b, return 1
		        return 1;
		    }
		    else
		    {
		        // if b should come after a, return -1
		        return -1;
		    }
		});

		// copy all properties so they are available on the view model
		for (var property in layoutModel) {

			if (property != 'width' & property != 'height' & property != 'zones')
				self[property] = layoutModel[property];
		}
	};

	return LayoutViewModel;
});

