define(['system/DateTime', "controllers/UserConfigurationController", "gui/controls/ComboBox", "gui/controls/Button", "gui/controls/Checkbox", "models/RotationCollection", "models/TimeZoneCollection", "system/remotecontrol/RemoteControlInterface", "properties/App.Settings", "system/io/File"], function (DateTime, UserConfigurationController, ComboBox, Button, Checkbox, RotationCollection, TimeZoneCollection, RemoteControlInterface, AppSettings, File) {
    /*
    	SetupView - TODO - Make this class generic and not SSSP Specific!
    	-------------------------------------------------------------------------
     */
    function SetupView(callbacks, isCancellable, remoteSettings) {
        var self = this;
        self.id = 'setupView';
        var initialised = false;

        if (isCancellable == undefined)
            isCancellable = true;

        var _callbacks = callbacks;

        var _hoverControlIndex = 0;
        var _selectedControl = null;
        var _controls = [];
        var _timeZoneComboBox;
        var _useNTPCheckbox;
        var _usePMModeForScreenManagement;
        var showSLRotationOptions;
        var _rotationComboBox;
        var _fwCheckbox;

        var zeroTouchConfigurationApplied;

        function init() {
            var dfd = $.Deferred();

            if (initialised) {
                return dfd.resolve();
            }

            initialised = true;

            checkConfigurationFileForZeroTouch().then(function (ztConfiguration) {
                zeroTouchConfigurationApplied = ztConfiguration;
                _timeZoneComboBox = new ComboBox('timeZoneComboBox', TimeZoneCollection, UserConfigurationController.timeZone());
                _controls.push(_timeZoneComboBox);

                if (AppSettings.application.sdkVersion >= 1.2) {
                    _useNTPCheckbox = new Checkbox("useNTPChecbox", UserConfigurationController.useNTP());
                    _controls.push(_useNTPCheckbox);
                }

                if (AppSettings.application.webOSVersion >= 4.1) {
                    _usePMModeForScreenManagement = new Checkbox("usePMModeCheckbox", UserConfigurationController.usePMModeForScreenManagement());
                    _controls.push(_usePMModeForScreenManagement);
                }

                showSLRotationOptions = (remoteSettings && remoteSettings.nativePortraitRotation && remoteSettings.nativePortraitRotation !== '0') ?
                    false :
                    true;

                if (showSLRotationOptions) {
                    _rotationComboBox = new ComboBox('rotationComboBox', RotationCollection, UserConfigurationController.rotation());
                    _controls.push(_rotationComboBox);
                }

                if (AppSettings.application.sdkVersion >= 1.2 && AppSettings.application.features.canPerformFirmwareUpdates) {
                    var fwUpdatesEnabled = false;

                    // If not manually configured use the default setting in the app settings
                    if (!UserConfigurationController.remoteFirmwareUpdates().configured) {
                        fwUpdatesEnabled = AppSettings.application.remoteFirmwareUpdates.enabled;
                    } else {
                        fwUpdatesEnabled = UserConfigurationController.remoteFirmwareUpdates().enabled;
                    }

                    _fwCheckbox = new Checkbox("fwChecbox", fwUpdatesEnabled);
                    _controls.push(_fwCheckbox);
                }

                var _saveButton = new Button('saveBtn', 'Save', {
                    click: function () {
                        saveSettings();
                    }
                });
                _controls.push(_saveButton);

                if (isCancellable) {
                    var _cancelButton = new Button('cancelBtn', 'Cancel', {
                        click: function () {
                            _callbacks.cancel();
                        }
                    });
                    _controls.push(_cancelButton);
                }

                // create html
                var viewHtml = "<div id='" + self.id + "' style='position:absolute;left:0px;top:0px;width:100%;height:100%;background-color:#FFF;text-align:center;z-index:30;'>";
                viewHtml += "<a id='" + self.id + "KeyFocus' href='javascript:void(0);'></a>";
                //viewHtml += "<img style='margin:20px;' src='images/sl_logo.png;' />";
                viewHtml += "<h1 style='color: #000; margin-bottom: 20px;'>Signagelive Settings</h1>";

                viewHtml += "<h2 style='color: #000;'>Time Zone:</h2>";
                viewHtml += _timeZoneComboBox.html;

                if (AppSettings.application.sdkVersion >= 1.2) {
                    viewHtml += "<h2 style='color: #000;'>Use NTP Server:</h2>";
                    viewHtml += _useNTPCheckbox.html;
                }

                if (AppSettings.application.webOSVersion >= 4.1) {
                    viewHtml += "<h2 style='color: #000;'>Use PM Mode:</h2>";
                    viewHtml += _usePMModeForScreenManagement.html;
                }

                if (showSLRotationOptions) {
                    viewHtml += "<h2 style='color: #000;'>Screen Rotation:</h2>";
                    viewHtml += _rotationComboBox.html;
                }

                if (AppSettings.application.sdkVersion >= 1.2 && AppSettings.application.features.canPerformFirmwareUpdates) {
                    viewHtml += "<h2 style='color: #000;'>Remote Firmware Updates</h2>";
                    viewHtml += _fwCheckbox.html;
                }

                if (zeroTouchConfigurationApplied) {
                    viewHtml += '<div style="text-align: center;width: 100%;height: 44px;font-size: 16px;color: black;"><span>This device has been setup remotely with zero touch configuration for "Use PM Mode", therefore local changes cannot be applied.</span></div>';
                }

                viewHtml += _saveButton.html;

                if (isCancellable)
                    viewHtml += _cancelButton.html;

                viewHtml += "</div>";

                self.html = viewHtml;
                dfd.resolve();
            });

            return dfd;
        };

        self.addedToCanvas = function () {
            init().then(function () {
                $('#canvas').append(self.html);

                // Enable key event processing
                $('#' + self.id + "KeyFocus").focus();

                $(_controls).each(function (i, control) {
                    control.addedToCanvas();
                });

                // select the first control
                _controls[0].hoverOn();

                $('#' + self.id + "KeyFocus").keydown(function () {
                    var keyCode = event.keyCode;

                    var keys = null;
                    try {
                        keys = RemoteControlInterface.keys;
                        switch (keyCode) {

                            // Handle Down/Right Press
                            case keys.KEY_DOWN:
                            case keys.KEY_RIGHT:
                                if (_selectedControl == null) {
                                    _controls[_hoverControlIndex].hoverOff();

                                    _hoverControlIndex++;

                                    if (_hoverControlIndex > _controls.length - 1) {
                                        _hoverControlIndex = 0;
                                    }
                                    _controls[_hoverControlIndex].hoverOn();
                                } else {
                                    _controls[_hoverControlIndex].handleKeyPress(keyCode);
                                }
                                break;

                                // Handle Up/Left Press
                            case keys.KEY_UP:
                            case keys.KEY_LEFT:
                                if (_selectedControl == null) {
                                    _controls[_hoverControlIndex].hoverOff();
                                    _hoverControlIndex--;

                                    if (_hoverControlIndex < 0) {
                                        _hoverControlIndex = _controls.length - 1;
                                    }
                                    _controls[_hoverControlIndex].hoverOn();
                                } else {
                                    _controls[_hoverControlIndex].handleKeyPress(keyCode);
                                }
                                break;

                                // Handle Enter
                            case keys.KEY_ENTER:
                                if (_controls[_hoverControlIndex] instanceof Button || _controls[_hoverControlIndex] instanceof Checkbox) {
                                    _controls[_hoverControlIndex].handleKeyPress(keyCode);
                                } else {
                                    if (_selectedControl == null) {
                                        _selectedControl = _controls[_hoverControlIndex];
                                        _selectedControl.select();
                                    } else {
                                        _selectedControl.deselect();
                                        _selectedControl = null;
                                    }
                                }
                                break;

                                // Back? No way to do this on LG
                            case keys.KEY_RETURN:

                                // Is this the back button?

                                break;
                        }

                    } catch (e) {
                        logger.error("Setup View: Error handling remote key press. " + e.message);
                    }

                });

                if (zeroTouchConfigurationApplied) {
                    $('#usePMModeCheckbox').prop("disabled", true);
                    $('#usePMModeCheckbox').addClass("disabled");
                }
            });
        };

        function saveSettings() {
            try {
                // settings..
                var settings = {};

                settings.timeZone = _timeZoneComboBox.selectedItem();
                // Only load rotation options if they are available. Otherwise just return current setting so that it doesnt change
                if (showSLRotationOptions) {
                    settings.rotation = _rotationComboBox.selectedItem();
                } else {
                    settings.rotation = UserConfigurationController.rotation();
                }

                if (AppSettings.application.sdkVersion >= 1.2) {
                    logger.info('Use NTP Checked: ' + _useNTPCheckbox.checked);
                    settings.useNTP = _useNTPCheckbox.checked;
                }

                settings.modules = {};

                // If webOS 3.0+ then save the PM Mode settings
                if (AppSettings.application.webOSVersion >= 4.1) {
                    logger.info('Use PM Mode Checked: ' + _usePMModeForScreenManagement.checked);
                    settings.usePMModeForScreenManagement = _usePMModeForScreenManagement.checked;
                };

                // Update GPIO - Hardcode to enabled
                if (AppSettings.application.modules.GPIOBasedInterrupts.supported) {
                    settings.modules.GPIOBasedInterrupts = {}
                    settings.modules.GPIOBasedInterrupts.enabled = true;
                    settings.modules.GPIOBasedInterrupts.options = {};
                    settings.modules.GPIOBasedInterrupts.options.trigger = {};
                    settings.modules.GPIOBasedInterrupts.options.trigger.type = AppSettings.application.modules.interactiveKiosk.options.trigger.type;

                }

                // Return remote firmware updates settings
                settings.remoteFirmwareUpdates = {};
                settings.remoteFirmwareUpdates.enabled = false;
                if (AppSettings.application.sdkVersion >= 1.2 && AppSettings.application.features.canPerformFirmwareUpdates) {
                    settings.remoteFirmwareUpdates.enabled = _fwCheckbox.checked;
                    settings.remoteFirmwareUpdates.configured = true;
                }

                // Can be reshown here
                $("#userMessage").css("visibility", "visible");

                _callbacks.complete(settings);
            } catch (e) {
                logger.error("Error trying to save updated settings");
            }
        };

        function checkConfigurationFileForZeroTouch() {
            var dfd = $.Deferred();
            try {
                // if < webOS 4.1 then do not check ZT configuration and just return false as currently we do not allow these players to be configured
                if (AppSettings.application.webOSVersion < 4.1) {
                    return dfd.resolve(false);
                }

                var lastConfigFilePath = AppSettings.application.fileSystem.systemFiles.lastConfigFilePath;
                if (File.exists(lastConfigFilePath) === false) {
                    logger.info('SetupView.checkConfigurationFileForZeroTouch(): No configuration available to load.');
                    return dfd.resolve(false);
                }
                // File exists
                File.read(lastConfigFilePath).done(function (data) {
                    // Load Config
                    var configuration = JSON.parse(data);
                    if (configuration.settings) {
                        // Zero Touch Configuration Available
                        dfd.resolve(true);
                    } else {
                        dfd.resolve(false);
                    }
                });
            } catch (e) {
                logger.error(e);
                dfd.resolve(false);
            }
            return dfd;
        };
    };

    return SetupView;
});