define(["gui/views/SVGConstants", "gui/PlayUrlHelper", "gui/views/SVGLayerBase", "gui/viewmodels/TextAnimationEnum", "models/MediaTypeConstants", "models/MimeTypeMediaTypeMappings", "properties/App.Settings", "system/io/File", "models/RssChannelModel", "controllers/UserConfigurationController", "models/RssChannelCache"], function(SVGConstants, PlayUrlHelper, SVGLayerBase, TextAnimation, MediaTypeConstants, MimeTypeMediaTypeMappings, Settings, File, RssChannel, userConfig, RssChannelCache) {
    function TextZoneView(zoneViewModel) {
        this.parent = new SVGLayerBase(zoneViewModel);
        for (var property in this.parent) {

            if (property != 'prototype' & property != 'parent')
                this[property] = this.parent[property];
        }

        var self = this;
        self.viewModel = zoneViewModel;

        this.itemPadding = 30;

        var textItems = [];

        var _currentItemIndex = 0;
        var _visibleItems = [];

        var animating = false;
        var animationRefreshRate = 1000 / 60;

        if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame;
        }

        self.play = function() {
            logger.debug('Starting TextZoneView');
            animating = true;

            // load the background - if there is one
            if (self.viewModel.backgroundAsset != null && MimeTypeMediaTypeMappings.getMediaType(self.viewModel.backgroundAsset.mimeType) == MediaTypeConstants.IMAGE) {
                self.bgElement = document.createElementNS(SVGConstants.XmlNS, "image");
                self.bgElement.setAttributeNS(null, "x", 0);
                self.bgElement.setAttributeNS(null, "y", 0);
                self.bgElement.setAttributeNS(null, "width", this.width);
                self.bgElement.setAttributeNS(null, "height", this.height);

                self.bgElement.setAttributeNS(null, "preserveAspectRatio", "none");

                PlayUrlHelper.getMediaAssetPlayUrl(self.viewModel.backgroundAsset).done(function(url) {
                    self.bgElement.setAttributeNS('http://www.w3.org/1999/xlink', "href", url);
                });

                this.svgElem.appendChild(self.bgElement);
            }

            if (window.requestAnimationFrame)
                window.requestAnimationFrame(animate);
            else
                animate();
        };

        self.stop = function() {
            logger.debug('Stopping TextZoneView');

            animating = false;
        };
        var opacity = 0;
        var isFadingOut = false;

        function animate() {
            if (animating == false)
                return;
            try {
                // are there any items in view?
                if (_visibleItems.length == 0) {

                    loadRSSFeed();

                    if (textItems.length > 0) {
                        logger.debug('Creating first crawl item ' + textItems[_currentItemIndex]);

                        self.createCrawlItem(textItems[_currentItemIndex]);
                    }
                }

                var addNew = false;
                var removeFirstItem = false;

                // do the animation
                $(_visibleItems).each(function(index, crawlItem) {

                    // handle fade in/out
                    if (self.viewModel.configurationParameters.animation == TextAnimation.FADE) {
                        animationRefreshRate = 1000 / 25;

                        try {
                            if (isFadingOut) {
                                opacity--;

                                if (opacity == 0) {
                                    isFadingOut = false;

                                    animationRefreshRate = 250;

                                    removeFirstItem = true;
                                    addNew = true;
                                }
                            } else {
                                opacity++;

                                if (opacity == 10) {
                                    isFadingOut = true;

                                    animationRefreshRate = 3000;
                                }
                            }

                            crawlItem.childNodes[0].setAttributeNS(null, 'fill-opacity', (opacity / 10).toFixed(1));
                        } catch (e) {
                            logger.error('Error setting opacity ' + e.message);
                        }
                    } else {

                        // handle animations.

                        var xTransform = 0;
                        var yTransform = 0;

                        var crawlItemWidth = getItemWidth(crawlItem);
                        var crawlItemHeight = getItemHeight(crawlItem);

                        var xforms = crawlItem.transform.baseVal;

                        // An SVGTransformList
                        var firstXForm;

                        // is there a transform list?
                        // if so get it as this is required to animate the text

                        firstXForm = xforms.getItem(0);

                        // An SVGTransform
                        if (firstXForm.type == SVGTransform.SVG_TRANSFORM_TRANSLATE) {
                            xTransform = firstXForm.matrix.e;
                            yTransform = firstXForm.matrix.f;
                        }

                        switch (self.viewModel.configurationParameters.animation) {
                            case TextAnimation.SCROLL_RIGHT_TO_LEFT:
                                xTransform--;

                                var addPosition = self.width - self.itemPadding - Math.floor(crawlItemWidth);

                                if (Math.floor(xTransform) == Math.floor(addPosition))
                                    addNew = true;

                                var removePosition = 0 - crawlItemWidth;
                                if (Math.floor(xTransform) == Math.floor(removePosition))
                                    removeFirstItem = true;
                                break;
                            case TextAnimation.SCROLL_LEFT_TO_RIGHT:
                                xTransform++;

                                if (Math.floor(xTransform) == Math.floor(self.itemPadding))
                                    addNew = true;

                                if (Math.floor(xTransform) == Math.floor(self.width))
                                    removeFirstItem = true;
                                break;
                            case TextAnimation.SCROLL_TOP_TO_BOTTOM:
                                yTransform++;
                                var addPosition = self.itemPadding;

                                if (Math.floor(yTransform) == Math.floor(addPosition))
                                    addNew = true;

                                var removePosition = self.height + self.itemPadding + Math.floor(crawlItemHeight);

                                if (Math.floor(yTransform) == Math.floor(removePosition))
                                    removeFirstItem = true;

                                break;
                            case TextAnimation.SCROLL_BOTTOM_TO_TOP:
                                yTransform--;

                                var addPosition = self.height - self.itemPadding - Math.floor(crawlItemHeight);

                                if (Math.floor(yTransform) == Math.floor(addPosition)) {
                                    addNew = true;
                                }

                                var removePosition = Math.floor(0 - Math.floor(crawlItemHeight));
                                if (Math.floor(yTransform) == removePosition)
                                    removeFirstItem = true;

                                break;
                            default:
                                xTransform--;

                                var addPosition = self.width - self.itemPadding - Math.floor(crawlItemWidth);
                                if (Math.floor(xTransform) == Math.floor(addPosition))
                                    addNew = true;

                                var removePosition = 0 - crawlItemWidth;
                                if (Math.floor(xTransform) == Math.floor(removePosition))
                                    removeFirstItem = true;
                                break;
                        }
                        firstXForm.setTranslate(xTransform, yTransform);
                    }
                });

                if (addNew) {
                    _currentItemIndex++;
                    if (_currentItemIndex == textItems.length) {
                        _currentItemIndex = 0;

                        // reload the feed
                        loadRSSFeed();
                    }
                    if (textItems.length > 0) {
                        self.createCrawlItem(textItems[_currentItemIndex]);
                    }

                    logger.debug('Text Zone Visible Items ' + _visibleItems.length);
                }

                if (removeFirstItem) {
                    self.svgElem.removeChild(_visibleItems.shift());

                    logger.debug('Text Zone Visible Items ' + _visibleItems.length);
                }

                if (animating) {
                    if (self.viewModel.configurationParameters.animation != TextAnimation.FADE && window.requestAnimationFrame) {
                        window.requestAnimationFrame(animate);
                    } else {
                        setTimeout(function() {
                            animate();
                        }, animationRefreshRate);
                    }
                }

            } catch (e) {
                logger.error('Error whilst animating TextZone ' + e.message);
            }
        }

        // This method can be used to get the width of a dom object, and it takes in to account whether the page has been rotated or not
        function getItemWidth(item) {
            //logger.debug('Is Rotated: ' + userConfig.isRotated() + ', Rotation Angle: ' + userConfig.rotationAngle());
            if (userConfig.isRotated() && (userConfig.rotationAngle() == 90 || userConfig.rotationAngle() == 270))
                return item.getBoundingClientRect().height;
            else
                return item.getBoundingClientRect().width;
        }

        // This method can be used to get the height of a dom object, and it takes in to account whether the page has been rotated or not
        function getItemHeight(item) {
            if (userConfig.isRotated() && (userConfig.rotationAngle() == 90 || userConfig.rotationAngle() == 270))
                return item.getBoundingClientRect().width;
            else
                return item.getBoundingClientRect().height;
        }

        function loadRSSFeed() {
            textItems = [];

            var rssAsset = null;
            // get the current playlist
            var currentPlaylist = getCurrentPlaylist();
            if (currentPlaylist != null) {
                $(currentPlaylist.playlistMediaAssets).each(function(i, playlistMediaAsset) {
                    if (playlistMediaAsset.mediaAsset.contentType == MediaTypeConstants.RSS) {
                        rssAsset = playlistMediaAsset.mediaAsset;
                    }
                });
            } else {
                rssAsset = self.viewModel.defaultMediaAsset;
            }

            if (Settings.application.features.canDownloadFiles) {
                loadFeedDataFromCache(rssAsset.externalId);
            } else {
                //TODO: make this url a setting
                var url = "https://proxy.signagelive.com/api/values?url=" + rssAsset.remoteUrl;
                $.ajax(url, {
                    async: false
                }).done(function(data, textStatus, jqXHR) {
                    for (var i = 0; i < data.Items.length; i++) {
                        textItems.push({
                            title: data.Items[i].Title
                        });
                    }
                });
            }

        }

        function loadFeedDataFromCache(assetId) {
            logger.debug('Loading feed from cache');

            try {
                var rssChannel = RssChannelCache[assetId];
                if (rssChannel != null) {
                    $(rssChannel.items).each(function(i, item) {
                        textItems.push(item);
                    });
                }
            } catch (e) {
                logger.error('Error loading RSS Channel from cache: ' + e.message);
            }
        }

        function loadFeedData(url) {
            try {
                // NOTE - need to use the AJAX call to load the xml as there have been instances where byte order mark is the first characted retured in the
                // xml jquery.parseXML does not handle this whereas ajax does.
                $.ajax({
                    url: url,
                    type: 'GET',
                    dataType: "xml",
                    async: false
                }).done(function(data, textStatus, jqXHR) {
                    logger.debug('RSS loaded ' + data);
                    if (data != null) {
                        var rssChannel = new RssChannel(data);
                        $(rssChannel.items).each(function(i, item) {
                            textItems.push(item);
                        });
                    }
                }).fail(function(jqXHR, textStatus, errorThrown) {
                    logger.error('Error loading RSS feed');
                });

            } catch (e) {
                logger.error('Error loading RSS Feed ' + e.message);
            }
        }

        function getCurrentPlaylist() {
            var currentPlaylist = null;
            $(self.viewModel.scheduledPlaylists).each(function(i, scheduledPlaylist) {
                try {
                    if (scheduledPlaylist.start <= DateTime.now() && scheduledPlaylist.end >= DateTime.now()) {
                        currentPlaylist = scheduledPlaylist.playlist;
                    }
                } catch (e) {

                }
            });

            if (currentPlaylist == null) {
                currentPlaylist = self.viewModel.defaultPlaylist;
            }

            return currentPlaylist;
        }


        self.createCrawlItem = function(text) {
            try {

                logger.debug('Creating item:' + text.title);

                var crawlItem = document.createElementNS(SVGConstants.XmlNS, "g");

                var textXOffset = 0;
                var textYOffset = 0;

                // load the background - if there is one
                // TODO implement seperators on FADE
                if (self.viewModel.seperatorAsset != null && self.viewModel.configurationParameters.animation != TextAnimation.FADE && self.viewModel.seperatorAsset.localFileName.indexOf('.gif') < 0) {
                    var seperatorImage = document.createElementNS(SVGConstants.XmlNS, "image");

                    switch (self.viewModel.configurationParameters.animation) {
                        case TextAnimation.SCROLL_RIGHT_TO_LEFT:
                        case TextAnimation.SCROLL_LEFT_TO_RIGHT:
                            seperatorImage.setAttributeNS(null, "x", 0);
                            seperatorImage.setAttributeNS(null, "y", 5);
                            textXOffset = this.height * 2 + this.itemPadding;
                            break;
                        case TextAnimation.SCROLL_TOP_TO_BOTTOM:
                        case TextAnimation.SCROLL_BOTTOM_TO_TOP:
                            var xPos = (this.width - (this.height * 2)) / 2;
                            seperatorImage.setAttributeNS(null, "x", xPos);
                            seperatorImage.setAttributeNS(null, "y", 0);
                            textYOffset = this.height + this.itemPadding;
                            break;
                        case TextAnimation.FADE:

                            break;
                    }
                    seperatorImage.setAttributeNS(null, "width", this.height * 2);
                    seperatorImage.setAttributeNS(null, "height", this.height - 10);
                    seperatorImage.setAttributeNS(null, "align", 'xMidYMax');

                    //PlayUrlHelper.getMediaAssetPlayUrl(self.viewModel.seperatorAsset).done(function(url) {
                    //	self.bgElement.setAttributeNS('http://www.w3.org/1999/xlink', "href", url);
                    //	crawlItem.appendChild(seperatorImage);
                    //});

                    var localFile = './content/' + self.viewModel.seperatorAsset.localFileName;
                    seperatorImage.setAttributeNS('http://www.w3.org/1999/xlink', "href", localFile);
                    crawlItem.appendChild(seperatorImage);
                }

                // add the text item to the crawl item
                var textElement = document.createElementNS(SVGConstants.XmlNS, "text");
                var textNode = document.createTextNode(text.title);
                textElement.appendChild(textNode);

                // configure the style properties
                textElement.setAttributeNS(null, 'font', self.viewModel.configurationParameters.font);
                textElement.setAttributeNS(null, 'font-size', self.viewModel.configurationParameters.font_size + 'px');
                if (self.viewModel.configurationParameters.font_color != undefined & self.viewModel.configurationParameters.font_color != null) {
                    var colourHex = self.viewModel.configurationParameters.font_color;
                    // compensate for SL bug that removes leading zeros on hex values!!!!!!!!
                    switch (colourHex.length) {
                        case 2:
                            colourHex = '#00000' + colourHex.substring(1);
                            break;
                        case 3:
                            colourHex = '#0000' + colourHex.substring(1);
                            break;
                        case 4:
                            colourHex = '#000' + colourHex.substring(1);
                            break;
                        case 5:
                            colourHex = '#00' + colourHex.substring(1);
                            break;
                        case 6:
                            colourHex = '#0' + colourHex.substring(1);
                            break;
                    }
                    textElement.setAttributeNS(null, 'fill', colourHex);
                } else
                    textElement.setAttributeNS(null, 'fill', '#000000');

                // add the text to the crawl item
                crawlItem.appendChild(textElement);

                // add to the parent svg element
                this.svgElem.appendChild(crawlItem);

                // calculate the width and height of the crawl item so it can be correctly positioned for the animation
                var crawlItemWidth = getItemWidth(crawlItem);
                var crawlItemHeight = getItemHeight(crawlItem);

                // calculate the size of the text so it can be positioned correclty for the animation
                var textWidth = getItemWidth(textElement);
                var textHeight = getItemHeight(textElement);

                //logger.debug("Text Width: " + textWidth + ", Text Height: " + textHeight);

                // set the position of the text element - taking into consideration the seperator image
                switch (self.viewModel.configurationParameters.animation) {
                    case TextAnimation.SCROLL_RIGHT_TO_LEFT:

                        // position the text item (taking into consideration if required the seperator)
                        textElement.setAttributeNS(null, "x", textXOffset);

                        // set the text so that it is vertically centered
                        textElement.setAttributeNS(null, "y", (this.height / 2) + (textHeight / 3));

                        // set the translation so that the item starts the animation off to the right hand side
                        crawlItem.setAttributeNS(null, 'transform', 'translate(' + (this.width) + ',' + 0 + ')');
                        break;
                    case TextAnimation.SCROLL_LEFT_TO_RIGHT:
                        // position the text item (taking into consideration if required the seperator)
                        textElement.setAttributeNS(null, "x", textXOffset);

                        // set the text so that it is vertically centered
                        textElement.setAttributeNS(null, "y", (this.height / 2) + (textHeight / 3));

                        // recalculate the crawl item width as it will potentially have changed if the text has been moved because of a seperator
                        crawlItemWidth = getItemWidth(crawlItem);

                        crawlItem.setAttributeNS(null, 'transform', 'translate(' + (0 - crawlItemWidth - this.itemPadding) + ',' + 0 + ')');
                        break;
                    case TextAnimation.SCROLL_TOP_TO_BOTTOM:
                        // check that the text will fit the box
                        var fontSize = self.viewModel.configurationParameters.font_size;
                        while (crawlItemWidth > this.width - 20) {
                            fontSize--;
                            textElement.setAttributeNS(null, 'font-size', fontSize + 'px');

                            crawlItemWidth = getItemWidth(crawlItem);
                        }

                        // re-calculate as this may have changed
                        textWidth = getItemWidth(textElement);
                        textHeight = getItemHeight(textElement);

                        // set the text so that it is horizontally centered
                        textElement.setAttributeNS(null, "x", (this.width / 2) - (textWidth / 2));

                        // position the text item (taking into consideration if required the seperator)
                        textElement.setAttributeNS(null, "y", textYOffset + textHeight);

                        // recalculate the crawl item width as it will potentially have changed if the text has been moved because of a seperator
                        crawlItemWidth = getItemWidth(crawlItem);
                        crawlItemHeight = getItemHeight(crawlItem);

                        crawlItem.setAttributeNS(null, 'transform', 'translate(' + 0 + ',' + (0 - crawlItemHeight) + ')');
                        logger.debug("Transform Attributes: translate(" + 0 + "," + (0 - crawlItemHeight) + ")");

                        break;
                    case TextAnimation.SCROLL_BOTTOM_TO_TOP:
                        // check that the text will fit the box
                        var fontSize = self.viewModel.configurationParameters.font_size;
                        while (crawlItemWidth > this.width - 20) {
                            fontSize--;
                            textElement.setAttributeNS(null, 'font-size', fontSize + 'px');

                            crawlItemWidth = getItemWidth(crawlItem);
                        }

                        // re-calculate as this may have changed
                        textWidth = getItemWidth(textElement);
                        textHeight = getItemHeight(textElement);

                        // set the text so that it is horizontally centered
                        textElement.setAttributeNS(null, "x", (this.width / 2) - (textWidth / 2));

                        // position the text item (taking into consideration if required the seperator)
                        textElement.setAttributeNS(null, "y", textYOffset + textHeight);

                        // set the initial transform
                        crawlItem.setAttributeNS(null, 'transform', 'translate(' + 0 + ',' + this.height + ')');
                        break;
                    case TextAnimation.FADE:
                        textElement.setAttributeNS(null, 'fill-opacity', 0.0);

                        // check that the text will fit the box
                        var fontSize = self.viewModel.configurationParameters.font_size;
                        while (crawlItemWidth > this.width - 20) {
                            fontSize--;
                            textElement.setAttributeNS(null, 'font-size', fontSize + 'px');

                            crawlItemWidth = getItemWidth(crawlItem);
                        }

                        // set the text so that it is horizontally centered
                        textElement.setAttributeNS(null, "x", (this.width / 2) - (crawlItemWidth / 2));
                        textElement.setAttributeNS(null, "y", (this.height / 2) + (crawlItemHeight / 2));
                        break;
                    default:
                        crawlItem.setAttributeNS(null, 'transform', 'translate(' + (this.width) + ',' + 0 + ')');
                        // set the text so that it is vertically centered
                        textElement.setAttributeNS(null, "y", (this.height / 2) + (self.viewModel.configurationParameters.font_size / 2));
                        break;
                }

                // add to the collection of crawl items
                _visibleItems.push(crawlItem);
            } catch (e) {
                logger.error('Error whilst creating text crawl elements ' + elements.message);
            }
        };
    }

    return TextZoneView;
});