var keys = {
	KEY_0: 48,
	KEY_1: 49,
	KEY_2: 50,
	KEY_3: 51,
	KEY_4: 52,
	KEY_5: 53,
	KEY_6: 54,
	KEY_7: 55,
	KEY_8: 56,
	KEY_9: 57,
};

var consoleAutoScroll = true;
var consoleScrollPos = 0;
var networkAutoScroll = true;
var networkScrollPos = 0;

(function() {
	// Configure debugger
	// Create debugger element
	var debuggerElem = document.createElement('div');
	debuggerElem.id = "debugger";

	// Create console element and sub elements and append to debugger
	var consoleElem = document.createElement('div');
	consoleElem.id = "console";

	var consoleHeaderElem = document.createElement('h2');
	consoleHeaderElem.appendChild(document.createTextNode('Console	-	[1 - Scroll Up / 4 - Scroll Down]'));
	consoleElem.appendChild(consoleHeaderElem);

	var consoleContentElem = document.createElement('div');
	consoleContentElem.id = "console-content";
	consoleElem.appendChild(consoleContentElem);

	debuggerElem.appendChild(consoleElem);
	// Create network element and sub elements and append to debugger
	var networkElem = document.createElement('div');
	networkElem.id = "network";

	var networkHeaderElem = document.createElement('h2');
	networkHeaderElem.appendChild(document.createTextNode('Network	-	[2 - Scroll Up / 5 - Scroll Down]'));
	networkElem.appendChild(networkHeaderElem);

	var networkContentElem = document.createElement('div');
	networkContentElem.id = "network-content";
	networkElem.appendChild(networkContentElem);

	debuggerElem.appendChild(networkElem);
	// Append to body
	document.body.appendChild(debuggerElem);

	// Set up key presses
	setTimeout(function() {
		//debuggerElem.focus();
		document.body.addEventListener('keydown', function(event) {

			var keyCode = event.keyCode;
			switch (keyCode) {
				case keys.KEY_1:
					scrollConsoleUp();
					break;
				case keys.KEY_4:
					scrollConsoleDown();
					break;
				case keys.KEY_2:
					scrollNetworkUp();
					break;
				case keys.KEY_5:
					scrollNetworkDown()
					break;
			}
		});
	}, 5000);
})();

function scrollConsoleUp() {
	var consoleEl = document.getElementById("console-content");
	consoleEl.scrollTop -= 75;
	consoleAutoScroll = false;
}

function scrollConsoleDown() {
	var consoleEl = document.getElementById("console-content");
	consoleEl.scrollTop += 300;
	if (consoleEl.scrollTop >= (consoleEl.scrollHeight - consoleEl.clientHeight)) {
		consoleAutoScroll = true;
	}
}

function scrollNetworkUp() {
	var networkEl = document.getElementById("network-content");
	networkEl.scrollTop -= 50;
	networkAutoScroll = false;
}

function scrollNetworkDown() {
	var networkEl = document.getElementById("network-content");
	networkEl.scrollTop += 100;
	if (networkEl.scrollTop >= (networkEl.scrollHeight - networkEl.clientHeight)) {
		networkAutoScroll = true;
	}
}

// Define a new console object
var console = (function(oldCons){
    return {
        log: function(text){
            oldCons.log(text);
            logConsoleMessageToDisplay('log', text);
        },
        info: function (text) {
            oldCons.info(text);
            logConsoleMessageToDisplay('info', text);
        },
        warn: function (text) {
            oldCons.warn(text);
            logConsoleMessageToDisplay('warn', text);
        },
        error: function (text) {
            oldCons.error(text);
            logConsoleMessageToDisplay('error', text);
        }
    };
}(window.console));
// Then redefine the old console
window.console = console;
// Function used to log to display
var consoleTraceCounter = 0;
var consoleTraceCounterMax = 2500;
function logConsoleMessageToDisplay(type, message) {
	var consoleEl = document.getElementById('console-content');
	// Remove older messages if needed
	if (consoleTraceCounter >= consoleTraceCounterMax) {
		consoleEl.removeChild(consoleEl.children[0]);
	}
	// Log new message
	var newSpanEl = document.createElement('span');
	newSpanEl.classList.add('console-message');
	newSpanEl.classList.add(type);
	var text = new Date().toLocaleTimeString() + '		:		' + type.toUpperCase() + '	:	' + message;
	newSpanEl.appendChild(document.createTextNode(text));
	consoleEl.appendChild(newSpanEl);
	consoleTraceCounter++;
}

(function() {
	var origOpen = XMLHttpRequest.prototype.open;
	var requestId = 0;
    XMLHttpRequest.prototype.open = function() {
		requestId++;
		this.arguments = arguments;
		this.requestId = requestId;
		this.requestStartTime = new Date();
        this.addEventListener('loadstart', logAJAXRequestToDisplay);
		this.addEventListener('load', logAJAXRequestToDisplay);
		this.addEventListener('loadend', logAJAXRequestToDisplay);
		this.addEventListener('progress', logAJAXRequestToDisplay);
		this.addEventListener('error', logAJAXRequestToDisplay);
		this.addEventListener('abort', logAJAXRequestToDisplay);
		this.addEventListener('timeout', logAJAXRequestToDisplay);
        origOpen.apply(this, arguments);
    };
})();

function logAJAXRequestToDisplay(evt) {
	var consoleEl = document.getElementById('network-content');
	var requestMethod = this.arguments[0];
	var requestUrl = this.arguments[1];
	var requestId = this.requestId;
	// Log new message
	var text = '';
	switch (evt.type) {
		case "loadstart":
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime, 'STARTED', evt.loaded, 'NA', 'NA', 'NA');
			break;
		case "load":
			// TODO
			break;
		case "loadend":
			var currentStatus = (this.status >= 200 && this.status < 400)
				? 'COMPLETED'
				: 'FAILED';
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime.toLocaleTimeString(), currentStatus, evt.loaded, this.status, this.statusText, new Date().toLocaleTimeString());
			break;
		case "progress":
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime.toLocaleTimeString(), 'IN PROGRESS', evt.loaded, 'NA', 'NA', 'NA');
			break;
		case "error":
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime.toLocaleTimeString(), 'ERROR', evt.loaded, this.status, this.statusText, new Date().toLocaleTimeString());
			this.error = true;
			break;
		case "abort":
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime.toLocaleTimeString(), 'ABORTED', evt.loaded, 0, 'ABORTED' , new Date().toLocaleTimeString());
			this.error = true;
			break;
		case "timeout":
			text = generateNetworkMessageText(requestId, requestMethod, requestUrl, this.requestStartTime.toLocaleTimeString(), 'TIMEOUT', evt.loaded, 0, 'TIMEOUT', new Date().toLocaleTimeString());
			this.error = true;
			break;
	}

	var className = evt.type;
	if (evt.type === 'loadend') {
		className = (this.status >= 200 && this.status < 400)
			? 'completed'
			: 'failed';
	}

	var requestElId = 'network-request-' + requestId;
	var requestEl = document.getElementById(requestElId);
	if (requestEl) {
		requestEl.innerHTML = text;
		requestEl.className = "";
		requestEl.classList.add('network-message');
		requestEl.classList.add(className);
	}
	else {
		var newSpanEl = document.createElement('span');
		newSpanEl.id = requestElId;
		newSpanEl.classList.add('network-message');
		newSpanEl.classList.add(evt.type);
		newSpanEl.appendChild(document.createTextNode(text));
		consoleEl.appendChild(newSpanEl);
	}
}

function generateNetworkMessageText(requestId, requestMethod, requestUrl, requestStartTime, currentStatus, bytesTransferred, responseStatus, responseStatusText, requestEndTime) {
	return 'Id: ' + requestId + ' | Method: ' + requestMethod + ' | URL: ' + requestUrl + ' | Status: ' + currentStatus + ' | Status: ' + responseStatus + ' ' + responseStatusText + ' | Start Time: ' + requestStartTime + ' | End Time: ' + requestEndTime + ' | Bytes Transferred: ' + bytesTransferred;
}

// Ensure output auto scrolls
setInterval(function() {	
	var consoleEl = document.getElementById("console-content");
	if (consoleEl && consoleAutoScroll === true) {
		consoleEl.scrollTop = consoleEl.scrollHeight - consoleEl.clientHeight
	}   
	var networkEl = document.getElementById("network-content");
	if (networkEl && networkAutoScroll === true) {
		networkEl.scrollTop = networkEl.scrollHeight - networkEl.clientHeight
	}   
}, 250);