require(["jquery", "utils/logger", "system/io/File", "system/io/DirectoryInfo", "system/io/Directory", "properties/App.Settings"], function($, logger, File, DirectoryInfo, Directory, Settings) {
    $(function() {

        // Show logging information
        logger.enableTrace = true;
        logger.level = 1;

        // Wait 10 seconds to connect debugger
        setTimeout(function() {

            try {

                // Inform user...
                logger.info("Starting Signagelive Removal");
                logger.info("Deleting Signagelive data files");

                // Delete settings files from local storage
                for (var key in localStorage) {

                    // Delete each file
                    if (key == Settings.application.fileSystem.systemFiles.settingsFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.propertiesFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.lastConfigFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.clientIdFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.clientKeyFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.appVersionFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.fileListFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    } else if (key == Settings.application.fileSystem.systemFiles.fileListUpgradeFilePath) {
                        localStorage.removeItem(key);
                        logger.info("Deleted: " + key);
                    }
                }

                // Delete Media Files
                logger.info("All Signagelive data files removed");
                logger.info("Deleting Signagelive media files");

                deleteMediaFiles()
                    .always(function() {
                        Directory.deleteDirectory("file://internal/data/")
                            .always(function() {
                                // Delete settings files from local storage
                                setTimeout(function() {
                                    localStorage.removeItem(Settings.application.fileSystem.systemFiles.fileListFilePath);
                                    logger.info("Deleted: " + Settings.application.fileSystem.systemFiles.fileListFilePath);
                                    // Confirm signagelive removal success
                                    logger.info("Signagelive removal complete");
                                }, 500);
                            });
                    })
                    .fail(function() {
                        // Confirm signagelive removal success
                        logger.error("The application encountered an error. Signagelive was removed but was unable to remove all temporary media files");
                    });
            } catch (e) {
                logger.error(e.message);
            }
        }, 1000);

        function deleteMediaFiles() {
            var dfd = $.Deferred();

            // get a list of all the files in the media directory
            var mediaDirPath = Settings.application.fileSystem.root + "/" + Settings.application.fileSystem.mediaDirectory;
            var mediaDirectory = new DirectoryInfo(mediaDirPath);
            var deletedFiles = 0;
            mediaDirectory.initialize()
                .done(function() {

                    logger.info('There are ' + mediaDirectory.files.length + ' files to be deleted.');

                    // if there are media files to delete them delete them
                    if (mediaDirectory.files.length > 0) {

                        $(mediaDirectory.files).each(function(i, file) {
                            try {

                                if (file.type != 'folder') {
                                    logger.info('Deleting: ' + file.path);

                                    file.delete()
                                        .done(function() {
                                            logger.info('Deleted: ' + file.path);
                                            deletedFiles++;

                                            if (file.path.indexOf('.wgt') != -1) {
                                                logger.debug('File was a widget. Deleting the widget files');
                                                //File is a widget - Delete the widget dir
                                                var start = file.path.lastIndexOf('/');
                                                //Get the guid preceded by a /
                                                var widgetName = file.path.substr(start).replace('.wgt', '');

                                                var widgetDir = Settings.application.fileSystem.widgetDirectory + widgetName;

                                                logger.debug('Deleting Widget Dir: ' + widgetDir);

                                                Directory.deleteDirectory(widgetDir)
                                                    .done(function() {
                                                        logger.info("Deleted directory: " + widgetDir);
                                                    })
                                                    .fail(function() {
                                                        logger.error("Error deleting directory: " + widgetDir);
                                                    });
                                            }

                                            // If we have deleted all files then resolve
                                            if (deletedFiles == mediaDirectory.files.length) {
                                                dfd.resolve();
                                            }
                                        })
                                        .fail(function() {
                                            deletedFiles++;
                                            logger.error('Error deleting ' + file.path + ' error: ' + e.message);

                                            // If we have deleted all files then resolve
                                            if (deletedFiles == mediaDirectory.files.length) {
                                                dfd.resolve();
                                            }

                                        });
                                } else {
                                    deletedFiles++;
                                    logger.info(file.path + ' is a sub-directory and will not be deleted');

                                    // If we have deleted all files then resolve
                                    if (deletedFiles == mediaDirectory.files.length) {
                                        dfd.resolve();
                                    }
                                }
                            } catch (e) {
                                deletedFiles++;
                                logger.error('Error deleting ' + file.name + ' error: ' + e.message);

                                // If we have deleted all files then resolve
                                if (deletedFiles == mediaDirectory.files.length) {
                                    dfd.resolve();
                                }
                            }
                        });
                    }
                    // Nothing to delete so resolve
                    else {
                        logger.info('No media files to delete.');
                        dfd.resolve();
                    }
                })
                .fail(function() {
                    logger.error('Error loading directory information for: ' + mediaDirPath + '. Cannot delete Signagelive media files.');
                    dfd.reject();
                });

            return dfd;
        }
    });
});