define(["system/DateTime", "models/ConnectivityModel", "models/ContentModel", "models/LoggingModel", "models/ScreenControlModel", "models/ClientDetailsModel", "models/ProofOfPlayConfigurationModel", "models/SynchronizationModel", "models/RealTimeEventsModel", "models/FirmwareUpdateModel", "models/RemoteSettingsModel","models/DataSyncServicesModel"], function(DateTime, ConnectivityModel, ContentModel, LoggingModel, ScreenControlModel, ClientDetailsModel, ProofOfPlayConfigurationModel, SynchronizationModel, RealTimeEventsModel, FirmwareUpdateModel, RemoteSettingsModel, DataSyncServicesModel) {
    function ConfigurationModel(data) {
        var self = this;

        self.changeId = data.change_id;
        self.lastModified = new Date(data.last_modified);
        self.connectivity = new ConnectivityModel(data.connectivity);

        self.realTimeEvents = [];

        if (data.real_time_events != null && data.real_time_events != undefined) {
            $(data.real_time_events).each(function(index, value) {
                var rte = new RealTimeEventsModel(value);
                self.realTimeEvents.push(rte);
            });
        }

        self.content = new ContentModel(data.content, self.realTimeEvents);

        self.screenControl = new ScreenControlModel(data.screen_control);
        self.synchronization = new SynchronizationModel(data.synchronization);

        self.logging = new LoggingModel(data.logging);
        self.clientDetails = new ClientDetailsModel(data.client_details);

        if (data.proof_of_play != null) {
            self.proofOfPlayConfig = new ProofOfPlayConfigurationModel(data.proof_of_play);
		}
		
		if (data.firmware_update != null) {
            self.firmwareUpdate = new FirmwareUpdateModel(data.firmware_update);
        }

        if (data.remote_settings != null) {
            self.remoteSettings = new RemoteSettingsModel(data.remote_settings);
        }

        if (data.data_sync_services != null) {
			self.dataSyncServices = new DataSyncServicesModel(data.data_sync_services);
		}
    }

    return ConfigurationModel;
});