define(["models/MediaTypeConstants", "models/ScheduleModel", "models/LayoutModel", "models/PlaylistModel", "models/MediaAssetModel", "models/InterruptModel", "models/RssChannelCache", "models/FeedModel", "models/IntegrationModel"], function(MediaTypeConstants, ScheduleModel, LayoutModel, PlaylistModel, MediaAssetModel, InterruptModel, RssChannelCache, FeedModel, IntegrationModel) {
    function ContentModel(data, globalRealTimeEvents) {
        var self = this;
        self.defaultPlaylistId = null;
        self.defaultPlaylist = null;
        self.schedules = [];
        self.interrupts = [];
        self.layouts = [];
        self.playlists = [];
        self.mediaAssets = [];
        self.feeds = [];
        self.integrations = [];
        
        $(data.integrations).each(function(index, value) {
			self.integrations.push(new IntegrationModel(value));
		});


        $(data.media_assets).each(function(index, value) {
            var mediaAsset = new MediaAssetModel(value, globalRealTimeEvents);
            self.mediaAssets.push(mediaAsset);

            if (mediaAsset.contentType == MediaTypeConstants.RSS) {
                RssChannelCache.loadRssAssetIntoCache(mediaAsset);
            }
            // If the mediaAsset integrationIds property has any values then
			// find the matching integration model and push it to mediaAsset.integrations.
			$(mediaAsset.integrationIds).each(function(index, integrationId) {
				// Get matching integration
				var matchingIntegrations = $.grep(self.integrations, function(integration) {
					return integration.externalId === integrationId;
				});
				if (matchingIntegrations.length === 1) {
					mediaAsset.integrations.push(matchingIntegrations[0]);
				}
			});
        });

        $(data.feeds).each(function(index, value) {
            var feed = new FeedModel(value);
            self.feeds.push(feed);

            $(feed.feedMediaAssets).each(function(index, feedMediaAsset) {
                // find the media asset
                var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                    return mediaAsset.externalId == feedMediaAsset.mediaAssetId;
                });

                if (mediaAssets.length == 1)
                    feedMediaAsset.mediaAsset = mediaAssets[0];
            });
        });

        $(data.playlists).each(function(index, value) {
            var playlist = new PlaylistModel(value);
            self.playlists.push(playlist);

            $(playlist.playlistMediaAssets).each(function(index, playlistMediaAsset) {
                // find the media asset
                var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                    return mediaAsset.externalId == playlistMediaAsset.mediaAssetId;
                });

                if (mediaAssets.length == 1) {
                    playlistMediaAsset.mediaAsset = mediaAssets[0];

                    //If it is a feed, save the feed on the playlist media asset
                    if (mediaAssets[0].contentType == MediaTypeConstants.FEED) {
                        var feeds = $.grep(self.feeds, function(feed, index) {
                            return feed.mediaAssetId == mediaAssets[0].externalId;
                        });

                        if (feeds.length == 1)
                            playlistMediaAsset.feed = feeds[0];
                    }
                }
            });
        });

        self.defaultPlaylistId = data.default_playlist_id;

        // find the default playlist
        var playlists = $.grep(self.playlists, function(playlist, index) {
            return playlist.externalId == self.defaultPlaylistId;
        });
        if (playlists.length == 1)
            self.defaultPlaylist = playlists[0];

        $(data.layouts).each(function(index, value) {
            var layout = new LayoutModel(value);
            self.layouts.push(layout);

            if (layout.backgroundAssetId != null) {
                // find the default media asset
                var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                    return mediaAsset.externalId == layout.backgroundAssetId;
                });
                if (mediaAssets.length == 1)
                    layout.backgroundAsset = mediaAssets[0];
            }

            $(layout.zones).each(function(index, zone) {
                // populate the background image
                var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                    return mediaAsset.externalId == zone.backgroundAssetId;
                });
                if (mediaAssets.length == 1)
                    zone.backgroundAsset = mediaAssets[0];

                if (zone.configurationParameters != null || zone.configurationParameters != undefined) {
                    if (zone.configurationParameters.seperator_asset_id != undefined) {
                        // populate the seperator image
                        var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                            return mediaAsset.externalId == zone.configurationParameters.seperator_asset_id;
                        });
                        if (mediaAssets.length == 1)
                            zone.seperatorAsset = mediaAssets[0];
                    }
                }

                // find the default media asset
                var mediaAssets = $.grep(self.mediaAssets, function(mediaAsset, index) {
                    return mediaAsset.externalId == zone.defaultMediaAssetId;
                });
                if (mediaAssets.length == 1)
                    zone.defaultMediaAsset = mediaAssets[0];

                // find the default playlist
                var playlists = $.grep(self.playlists, function(playlist, index) {
                    return playlist.externalId == zone.defaultPlaylistId;
                });
                if (playlists.length == 1)
                    zone.defaultPlaylist = playlists[0];

                // find the scheduled playlists
                $(zone.scheduledPlaylists).each(function(index, scheduledPlaylist) {
                    var playlists = $.grep(self.playlists, function(playlist, index) {
                        return playlist.externalId == scheduledPlaylist.playlistid;
                    });

                    if (playlists.length == 1)
                        scheduledPlaylist.playlist = playlists[0];
                });
            });
        });

        $(data.schedules).each(function(index, value) {
            var schedule = new ScheduleModel(value);
            self.schedules.push(schedule);

            // find the layouts
            var layouts = $.grep(self.layouts, function(layout, index) {
                return schedule.layoutId == layout.externalId;
            });
            if (layouts.length == 1)
                schedule.layout = layouts[0];
        });

        try {
            $(data.interrupts).each(function(index, value) {
                var interrupt = new InterruptModel(value);
                self.interrupts.push(interrupt);

                // find the layouts
                var layouts = $.grep(self.layouts, function(layout, index) {
                    return interrupt.action.layoutId == layout.externalId;
                });
                if (layouts.length == 1)
                    interrupt.action.layout = layouts[0];
            });
        } catch (e) {
            logger.error("Error whilst loading interrupts: " + e.message);
        }
    }

    return ContentModel;
});