define(["models/ValidityModel", "models/PlaylistMediaAssetActionModel"], function(ValidityModel, PlaylistMediaAssetActionModel) {
	function FeedMediaAssetModel(data) {
		var self = this;
		self.externalId = data.id;
		self.mediaAssetId = data.media_asset_id;
		self.duration = data.duration * 1000;
		self.includeInProofOfPlay = data.include_in_proof_of_play;
		self.actions = null;
        self.popAssetID = data.pop_asset_id;
        self.grouping = data.grouping;

		if (data.actions != null)
		{
			self.actions = [];

			$.each(data.actions, function(index, action)
			{
				self.actions[index] = new PlaylistMediaAssetActionModel(action);
			});
		}
		
		self.validity = null;
		if (data.validity != null)
			self.validity = new ValidityModel(data.validity);

		self.mediaAsset = null;

		self.isCurrentlyValid = function()
        {
			if (self.mediaAsset.hasFailed)
			{
                logger.debug('Asset has failed - failure to play');
                return false;
            }
            if (self.validity == null)
            {
                logger.debug('No validity on PMA, checking mediaasset');
				return self.mediaAsset.isCurrentlyValid();
            } else
            {
                return self.validity.isCurrentlyValid();
            }
		};

        self.getValidPeriods = function(startDate, endDate)
        {
            var periods = [];

            logger.info('Calculating validity periods for asset');

            // no valid - the asset is therefore valid the entire time specified
            if (self.validity == null) {
                logger.info('No validity set - play all the time');
                periods.push({
                    start : startDate,
                    end : endDate
                });
            } else {
                var validityStart = (self.validity.start == null) ? startDate : self.validity.start;
                var validityEnd = (self.validity.end == null) ? endDate : self.validity.end;

                // calc the periods the asset will recur
                if (self.validity.days == null) {
                    // not recurrence therefore valid all day every day for the validity period
                    periods.push({
                        start : validityStart,
                        end : validityEnd
                    });
                } else {
                    // the asset has recurrence - calculate the periods it will play for the specified period

                    // calculate the recurrence for each day
                    var dateToCheck = startDate;
                    while (dateToCheck <= endDate) {

                        // is the asset valid on the specified day?
                        var isValidOnDayOfWeek = false;
                        var dayNumber = dateToCheck.getDay();
                        switch(dayNumber) {
                            case 0:
                                // Sunday
                                isValidOnDayOfWeek = ((self.validity.days & 1) == 1);
                                break;
                            case 1:
                                // Monday
                                isValidOnDayOfWeek = ((self.validity.days & 2) == 2);
                                break;
                            case 2:
                                // Tuesday
                                isValidOnDayOfWeek = ((self.validity.days & 4) == 4);
                                break;
                            case 3:
                                // Wednesday
                                isValidOnDayOfWeek = ((self.validity.days & 8) == 8);
                                break;
                            case 4:
                                // Thursday
                                isValidOnDayOfWeek = ((self.validity.days & 16) == 16);
                                break;
                            case 5:
                                // Friday
                                isValidOnDayOfWeek = ((self.validity.days & 32) == 32);
                                break;
                            case 6:
                                // Saturday
                                isValidOnDayOfWeek = ((self.validity.days & 64) == 64);
                                break;
                        }

                        // if the asset is valid on the day of the week - calculate the exact times
                        if (isValidOnDayOfWeek) {
                            if (self.validity.startTime != null & self.validity.endTime != null) {
                                var calculatedRecurrenceStart = dateToCheck.setTime(self.validity.startTime);
                                var calculatedRecurrenceEnd = dateToCheck.setTime(self.validity.endTime);

                                // handle overnight validity (i.e. end is before the start)
                                if (calculatedRecurrenceEnd < calculatedRecurrenceStart)
                                    calculatedRecurrenceEnd = calculatedRecurrenceEnd.addDays(1);

                                // adjust date time to ensure that assets don't play outside of the overal validity
                                calculatedRecurrenceStart = (calculatedRecurrenceStart < validityStart) ? validityStart : calculatedRecurrenceStart;
                                calculatedRecurrenceEnd = (calculatedRecurrenceEnd > validityEnd) ? validityEnd : calculatedRecurrenceEnd;

                                // add to the list of times the
                                periods.push({
                                    start : calculatedRecurrenceStart,
                                    end : calculatedRecurrenceEnd
                                });
                            }
                        }

                        // move to the next day!
                        dateToCheck = dateToCheck.addDays(1);
                    }
                }
            }

            return periods;
        };
	}

	return FeedMediaAssetModel;
});
