define(["system/DateTime", "models/MediaTypeConstants", "models/FeedMediaAssetModel"], function(DateTime, MediaTypeConstants, FeedMediaAssetModel) {
	function FeedModel(data)
	{
		var self = this;
		self.externalId = 0;
		self.mediaAssetId = 0;
		self.name = '';
		self.playMode = '';

		self.feedMediaAssets = [];
		self.externalId = data.id;
		self.name = data.name;
		self.mediaAssetId = data.media_asset_id;
		self.playMode = data.play_mode;
		self.includeInProofOfPlay = data.include_in_proof_of_play;
		self.assetsToPlay = 0; //0 means play them all!

		$(data.media_assets).each(function(index, value) {
			var feedMediaAsset = new FeedMediaAssetModel(value);
			self.feedMediaAssets.push(feedMediaAsset);
		});

		if(data.meta_data)
		{
			if(data.meta_data.assets_to_play)
			{
				self.assetsToPlay = parseInt(data.meta_data.assets_to_play);
			}
		}

		if(self.playMode === "reverse")
			self.feedMediaAssets = self.feedMediaAssets.reverse();

		self.isAllVideos = function() {
			var isAllVideos = true;

			for (var i = 0; i < self.feedMediaAssets.length; i++) {
				var plma = self.feedMediaAssets[i];
				var ma = plma.mediaAsset;

				if (ma.contentType == MediaTypeConstants.VIDEO) {
					isAllVideos = isAllVideos & true;
				} else {
					isAllVideos=false;
					break;
				}
			}

			return isAllVideos;
		};

		self.randomize = function()
		{
			logger.debug("Randomizing feed");
			var tempAssetsArray = [];
			var currentGrouping = -1;
			var tempGroupArray = [];

			//Preserve groupings
			$(self.feedMediaAssets).each(function(index, feedMediaAsset) {
				if(currentGrouping != feedMediaAsset.grouping)
				{
					if(tempGroupArray.length > 0)
					{

						if(tempGroupArray.length > 1)
						{		
							//if we have more than 1 asset, randomize the group array before adding it to the list
							randomizeArray(tempGroupArray);
						}
						
						//Push to the array of arrays
						tempAssetsArray.push(tempGroupArray.slice(0));
						tempGroupArray = [];
					}
					currentGrouping = feedMediaAsset.grouping;
				}

				tempGroupArray.push(feedMediaAsset);
			});

			//Add the last group to the array
			if(tempGroupArray.length > 0)
			{
				if(tempGroupArray.length > 1)
				{
					//if we have more than 1 asset, randomize the group array before adding it to the list
					randomizeArray(tempGroupArray);
				}
				
				//Push to the array of arrays
				tempAssetsArray.push(tempGroupArray.slice(0));
			}
	
			//Randomize the array of arrays
			randomizeArray(tempAssetsArray);
			
			//Re-add to feedmediaassets
			self.feedMediaAssets = [];
			$(tempAssetsArray).each(function(index, array) {
				$(array).each(function(index2, feedMediaAsset) {	
					self.feedMediaAssets.push(feedMediaAsset);			
				});			
			});

		}

	}

	function randomizeArray(array)
	{
		for (var i = array.length - 1; i > 0; i--) {
			var j = Math.floor(Math.random() * (i + 1));
			var temp = array[i];
			array[i] = array[j];
			array[j] = temp;
		}
	}

	return FeedModel;
});
