define(["properties/App.Settings", "models/StorageTypeConstants", "system/io/FileBase"], function(Settings, StorageType, FileBase) {
    /**
     * Used to manage the Local File List Manifest
     */
    function LocalFileList() {

        var self = this;
        self.files = [];

        self.initialize = function() {
            var dfd = $.Deferred();
            if (Settings.application.sdkVersion >= 1.2 && Settings.application.packageType === "IPK") {
                FileBase.readFromFileSystem(Settings.application.fileSystem.systemFiles.fileListFilePath)
                    .done(function(data) {
                        if (data) {
                            try {
                                self.files = JSON.parse(data);
                            } catch (e) {}
                        }
                        dfd.resolve();
                    })
                    .fail(function() {
                        logger.error('Error reading file list');
                        dfd.reject();
                    });
            } else {
                // Load the local manifest file
                if (localStorage[Settings.application.fileSystem.systemFiles.fileListFilePath] != undefined) {
                    self.files = JSON.parse(localStorage[Settings.application.fileSystem.systemFiles.fileListFilePath]);
                    // Do we need to upgrade the paths for the new version of the app?
                    if (localStorage[Settings.application.fileSystem.systemFiles.fileListUpgradeFilePath] == undefined) {
                        // This code here is requried to ensure that all existing files stored in the file list path are updated correctly
                        var pathsUpdated = false;
                        self.files.forEach(function(file) {
                            if (file.path.lastIndexOf("file://internal/procentric/scap/application/app/content", 0) === 0) {
                                file.path = file.path.replace("file://internal/procentric/scap/application/app/content", "file://internal");
                                pathsUpdated = true;
                            }
                        });

                        if (pathsUpdated) {
                            saveFileSystemEnties();
                        }
                        setFileListUpgraded(true);
                    }
                }
                setTimeout(function() {
                    dfd.resolve();
                }, 50);
            }
            return dfd;
        };

        /**
         * Add a file to the file manifest
         * @param {[type]} path [description]
         */
        self.add = function(path) {
            var dfd = $.Deferred();
            // If the file does not exist then add it and save it
            if (self.exists(path) == false) {

                self.files.push({
                    path: path,
                });
                saveFileSystemEnties().always(function() {
                    dfd.resolve();
                });
            } else {
                setTimeout(function() {
                    dfd.resolve();
                }, 100);
            }
            return dfd;
        };

        /**
         * Remove a file from the file manifest
         * @param  {[type]} path [description]
         * @return {[type]}      [description]
         */
        self.remove = function(path) {
            var dfd = $.Deferred();
            var index = -1;
            indexes = $.map(self.files, function(obj, index) {
                if (obj.path == path) {
                    return index;
                }
            });

            if (indexes.length > 0)
                index = indexes[0];

            if (index != -1) {
                self.files.splice(index, 1);
                saveFileSystemEnties().always(function() {
                    dfd.resolve();
                });
            } else {
                setTimeout(function() {
                    dfd.resolve();
                }, 100);
            }
            return dfd;
        };

        /**
         * Does the file exist?
         * @param  {[type]} path [description]
         * @return {[type]}      [description]
         */
        self.exists = function(path) {
            var exists = false;
            var matches = $.grep(self.files, function(obj, index) {
                return obj.path == path;
            });
            return (matches.length == 1);
        };

        /**
         * [saveFileSystemEnties description]
         */
        function saveFileSystemEnties() {
            var dfd = $.Deferred();
            if (Settings.application.sdkVersion >= 1.2 && Settings.application.packageType === "IPK") {
                FileBase.writeToFileSystem(Settings.application.fileSystem.systemFiles.fileListFilePath, JSON.stringify(self.files)).done(function() {
                    dfd.resolve();
                }).fail(function() {
                    dfd.reject();
                });
            } else {
                setTimeout(function() {
                    localStorage[Settings.application.fileSystem.systemFiles.fileListFilePath] = JSON.stringify(self.files);
                    dfd.resolve();
                }, 100);
            }
            return dfd;
        }

        /**
         * [setFileListUpgraded description]
         */
        function setFileListUpgraded(updated) {
            var dfd = $.Deferred();
            if (Settings.application.sdkVersion >= 1.2 && Settings.application.packageType === "IPK") {
                FileBase.writeToFileSystem(Settings.application.fileSystem.root + "/" + Settings.application.fileSystem.systemFiles.fileListUpgradeFilePath, JSON.stringify({ updated: updated })).done(function() {
                    dfd.resolve();
                }).fail(function() {
                    dfd.reject();
                });
            } else {
                setTimeout(function() {
                    localStorage[Settings.application.fileSystem.systemFiles.fileListUpgradeFilePath] = JSON.stringify({ updated: updated });
                    dfd.resolve();
                }, 100);
            }
            return dfd;
        }
    }
    var lfs = new LocalFileList();
    return lfs;
});