define(["models/MimeTypeMediaTypeMappings", "models/MediaTypeConstants", "models/ValidityModel", "models/RealTimeEventsModel"], function(MimeTypeMediaTypeMappings, MediaTypeConstants, ValidityModel, RealTimeEventsModel) {
    function MediaAssetModel(data, globalRealTimeEvents) {
        var self = this;
        self.externalId = data.id;
        self.name = data.name;
        self.mimeType = data.type;
        self.contentType = MimeTypeMediaTypeMappings.getMediaType(self.mimeType);
        self.remoteUrl = data.url;
        self.localFileName = data.local_file_name;
        self.hash = data.hash;
        self.hashType = data.hashType;
        self.size = data.size;
        self.isDownloadable = (self.contentType == MediaTypeConstants.IMAGE || self.contentType == MediaTypeConstants.VIDEO || self.contentType == MediaTypeConstants.RSS || self.contentType == MediaTypeConstants.WIDGET);
        self.isExternallyDelivered = (self.contentType == MediaTypeConstants.RSS);
        self.hasFailed = false;
        self.numberOfFailures = 0;
        self.includeInProofOfPlay = data.include_in_proof_of_play;
        self.popAssetID = data.pop_asset_id;
        self.metaData = data.meta_data;
        self.integrationIds = data.integrations;
        self.integrations = []; // These are set at the ContentModel level

        self.realTimeEvents = [];

        if (data.real_time_events != null && data.real_time_events != undefined) {

            if (data.real_time_events.global != null && data.real_time_events.global != undefined) {

                // need to get the RTE object from global configuration
                $(data.real_time_events.global).each(function(index, globalRealTimeEventId) {

                    // find the id on the global list
                    $(globalRealTimeEvents).each(function(index, globalRealTimeEvent) {
                        if (globalRealTimeEventId == globalRealTimeEvent.id) {
                            self.realTimeEvents.push(globalRealTimeEvent);
                        }
                    });
                });
            }

            // add custom RTEs
            if (data.real_time_events.custom != null && data.real_time_events.custom != undefined) {
                $(data.real_time_events.custom).each(function(index, customRTEConfig) {
                    var customRTE = new RealTimeEventsModel(customRTEConfig);
                    self.realTimeEvents.push(customRTE);
                });
            }
        }

        self.duration = 0;
        if (data.duration != null) {
            self.duration = data.duration;
        }

        self.validity = null;
        if (data.validity != null)
            self.validity = new ValidityModel(data.validity);

        self.isCurrentlyValid = function(date) {
            if (self.validity == null)
                return true;
            else
                return self.validity.isCurrentlyValid(date);
        };
    }

    return MediaAssetModel;
});