define(["system/DateTime", "models/ScreenControlPlanModel"], function(DateTime, ScreenControlPlanModel) {
	function ScreenControlModel(data) {
		var self = this;
		self.plans = [];
		self.persistedState = false; //this is used when to use WBT API screen control settings that override default behaviour

		if (data != null) {
			$(data.plans).each(function(index, value) {
				var screenControlPlan = new ScreenControlPlanModel(value);
				self.plans.push(screenControlPlan);
			});
		}

		function getScreenControlSchedule() {
			var schedules = [];

			var scheduleStartDate = DateTime.now().setTime('00:00:00').addDays(-1);
			var scheduleEndDate = scheduleStartDate.addDays(8);

			var currentDateBeingScheduled = scheduleStartDate;

			var lastOffTime = currentDateBeingScheduled;

			// add all of the 'on' periods to the schedule
			while (currentDateBeingScheduled < scheduleEndDate) {

				// get the plans relavent to the date being scheduled?
				// a plan is a 'on' period
				var relaventPlans = $.grep(self.plans, function(plan, index) {
					return plan.isActiveOnDate(currentDateBeingScheduled);
				});

				// sort the plans by time order
				relaventPlans = relaventPlans.sort(function(a, b) {
					if (a.onTime > b.onTime)
						return 1;
					if (a.onTime < b.onTime)
						return -1;
					return 0;
				});

				var now = DateTime.now();

				$(relaventPlans).each(function(i, plan) {

					// calculate the on and off times
					var onTime = currentDateBeingScheduled.setTime(plan.onTime);
					var offTime = currentDateBeingScheduled.setTime(plan.offTime);

					// if the offtime is 'earlier' than the on time it is intended for the next day
					if (offTime < onTime)
						offTime = offTime.addDays(1);

					if (onTime > lastOffTime) {
						schedules.push({
							time : lastOffTime,
							action : 'off'
						});
					}
					schedules.push({
						time : onTime,
						action : 'on'
					});

					lastOffTime = offTime;
				});

				currentDateBeingScheduled = currentDateBeingScheduled.addDays(1);
			}

			// add the last off time as the last command to execute
			schedules.push({
				time : lastOffTime,
				action : 'off'
			});

			return schedules;
		}


		self.nextAction = function() {
			var nextAction = null;
			if (self.plans.length > 0) {

				var schedules = getScreenControlSchedule();

				var now = DateTime.now();

				for (var i = 0; i < schedules.length; i++) {
					if (schedules[i].time > now) {
						nextAction = schedules[i];
						break;
					}
				}
			}
			return nextAction;
		};

		self.isScreenCurrentlyScheduledOn = function(screenInformation) {
			// if there are no screen control plans the screen should be on
			// if there are screen control plans the screen is off anytime there is no valid on period

			var isCurrentlyScheduledOn = false;
			var screenStatus = (screenInformation) ? screenInformation.screenStatus : null;
			var persistedScreenStatusDT = (screenInformation) ? new Date(screenInformation.screenStatusUpdatedDateTime) : null;

			if (self.plans.length > 0) {
				var schedules = getScreenControlSchedule();

				var now = DateTime.now();

				var activeSchedule = null;

				for (var i = 0; i < schedules.length; i++) {
					if (schedules[i].time > now) {

						// the active schedule is the previous one in the array
						activeSchedule = schedules[i - 1];
						break;
					}
				}

				if (activeSchedule != null) {
					// if the active schedules start time is before the persisted screen state then that means the persisted state takes precendence until there is a active schedule change afterwards
					if(activeSchedule.time < persistedScreenStatusDT) {
						self.persistedState = true;
						isCurrentlyScheduledOn = (screenStatus == "ScreenOff") ? false : true;
					} else { //else we should just follow the active schedule and clear the previous persisted state
						self.persistedState = false;
						isCurrentlyScheduledOn = (activeSchedule.action == 'on');
					}
				}

			} else {
				// Else no screen control plans - by default the screen should be on all the time but w/ the WBT API screen control state persistence this would need to be checked
				if(screenStatus) {
					self.persistedState = true;
					isCurrentlyScheduledOn = (screenStatus == "ScreenOff") ? false : true;
				} else {
					self.persistedState = false;
					isCurrentlyScheduledOn = true;
				}
			}

			return isCurrentlyScheduledOn;
		};

		self.isScreenCurrentlyScheduledOff = function() {
			return (!self.isScreenCurrentlyScheduledOn());
		};
	}

	return ScreenControlModel;
});
