define(['models/SystemReport_SystemModel', 'models/SystemReport_ClientModel', 'models/SystemReport_CPUModel', 'models/SystemReport_MemoryModel', 'models/SystemReport_StorageDeviceModel', 'models/SystemReport_TemperatureModel', 'models/SystemReport_GraphicsDeviceModel', 'models/SystemReport_NetworkAdapterModel', 'models/SystemReport_SystemSettingsModel', 'models/UserConfigurationModel'], function(SystemReport_SystemModel, SystemReport_ClientModel, SystemReport_CPUModel, SystemReport_MemoryModel, SystemReport_StorageDeviceModel, SystemReport_TemperatureModel, SystemReport_GraphicsDeviceModel, SystemReport_NetworkAdapterModel, SystemReport_SystemSettingsModel, UserConfigurationModel) {
	/*
		System Report Model - Provides a model for Signagelive System Reports
		-------------------------------------------------------------------------
		Use the generate method to 
	 */
	function SystemReportModel() {
		var self = this;

		//
		// Generates the system report
		//
		self.generate = function() {
			var dfd = $.Deferred();
			logger.info("Starting system report generation...");

			// System report properties
			self.system = null;
			self.client = null;
			self.cpus = [];
			self.memory = null;
			self.storage_devices = [];
			self.temperature = [];
			self.graphics_devices = [];
			self.network_adapters = [];
			self.installed_applications = [];
			self.running_processes = [];
			self.app_settings = {};
			self.system_settings = {};

			// System Report Models
			var _system = new SystemReport_SystemModel();
			var _client = new SystemReport_ClientModel();
			var _cpu = new SystemReport_CPUModel();
			var _memory = new SystemReport_MemoryModel();
			var _storage = new SystemReport_StorageDeviceModel();
			var _temperature = new SystemReport_TemperatureModel();
			var _graphics = new SystemReport_GraphicsDeviceModel();
			var _network = new SystemReport_NetworkAdapterModel();
			var _systemSettings = new SystemReport_SystemSettingsModel();

			// Generate system report
			logger.info("Generating system information...");
			_system.generate().done(function() {
				logger.info("System information generated.");
				self.system = _system;
			}).always(function() {
				// Generate client report
				logger.info("Generating system information...");
				_client.generate().done(function() {
					logger.info("Client information generated.");
					self.client = _client;
				}).always(function() {
					// Generate CPU report
					logger.info("Generating cpu information...");
					_cpu.generate().done(function() {
						logger.info("CPU information generated.");
						self.cpus.push(_cpu);
					}).always(function() {
						// Generate memory report
						logger.info("Generating memory information...");
						_memory.generate().done(function() {
							logger.info("Memory information generated.");
							self.memory = _memory;
						}).always(function() {
							// Generate Temperature report
							logger.info("Generating temperature information...");
							_temperature.generate().done(function() {
								logger.info("Temperature information generated.");
								self.temperature.push(_temperature);
							}).always(function() {
								// Generate graphics report
								logger.info("Generating graphics information...");
								_graphics.generate().done(function() {
									logger.info("Graphics information generated.");
									self.graphics_devices.push(_graphics);
								}).always(function() {
									// Generate network report
									logger.info("Generating network information...");
									_network.generate().done(function() {
										logger.info("Network information generated.");
										self.network_adapters = [_network];
									}).always(function() {
										// Generate system settings report
										logger.info("Generating system settings information...");
										_systemSettings.generate().done(function() {
											logger.info("System settings information generated.");
											self.system_settings = _systemSettings;
										}).always(function() {
											// Generate storage report
											logger.info("Generating storage information...");
											_storage.generate().done(function() {
												logger.info("Storage information generated.");
												self.storage_devices.push(_storage);
											}).always(function() {
												logger.info("All System information generated.");
												// Instantiate other reporting bits
												self.app_settings = {};
												self.installed_applications = [];
												self.running_processes = [];
												// Resolve system report generation
												dfd.resolve();
												
											});
										});
									});
								});
							});
						});
					});
				});
			});

			return dfd;
		};
	}

	return SystemReportModel;
});

