define(["system/cpu/CPUInterface"], function(CPUInterface) {
	/*
		CPU Model - Provides a model for Device CPU information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/cpu/CPUInterface class
		which provides access to lower level cpu information for the device.
	 */
	function SystemReport_CPUModel() {
		var self = this;
		self.name = 'Unknown';
		self.speed = 1000;
		self.usage = 0;

		//
		// Attempts to populate the CPU Model with accurate data
		//
		self.generate = function() {
			var dfd = $.Deferred();
			logger.info("Getting CPU Usage..");
			// Get Usage data
			CPUInterface.getCPUUsage().done(function(cpuUsage) {
				self.usage = cpuUsage;
			}).always(function() {
				// Get CPU Speed
				logger.info("Getting CPU Speed..");
				CPUInterface.getCPUSpeed().done(function(cpuSpeed) {
					self.speed = cpuSpeed;
				}).always(function() {
					// Get CPU Name
					logger.info("Getting CPU Name..");
					CPUInterface.getName().done(function(name){
						self.name = name;
					}).always(function() {
						// Resolve CPU Info
						logger.info("Finished loading CPU info");
						dfd.resolve();
					});
				});
			});

			return dfd;
		};
	}

	return SystemReport_CPUModel;
});
