define(["system/graphics/GraphicsInterface"], function(GraphicsInterface) {
	/*
		Graphics Model - Provides a model for Device temperature information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/graphics/GraphicsInterface class
		which provides access to lower level graphics information for the device.
	 */
	function SystemReport_GraphicsDeviceModel() {
		var self = this;

		// Defaults
		self.name = 'Internal Graphics';
		self.current_resolution_horizontal = document.documentElement.clientWidth;
		self.current_resolution_vertical = document.documentElement.clientHeight;

		//
		// Attempts to populate the graphics model
		//
		self.generate = function() {

			var dfd = $.Deferred();

			logger.info('Generate Graphics Information');

			// Get graphics name
			logger.info("Getting graphics name...");
			GraphicsInterface.getGraphicsName().done(function(gName){
				self.name = gName;
			}).always(function(){

				// Get horizontal resolution
				logger.info("Getting resolution..");
				GraphicsInterface.getGraphicsHorizontalResolution().done(function(hRes) {
					self.current_resolution_horizontal = hRes;
				}).always(function() {

					// Get vertical resolution
					GraphicsInterface.getGraphicsVerticalResolution().done(function(vRes){
						self.current_resolution_vertical = vRes;
					}).always(function() {

						// Resolve graphics system model generation
						logger.info("Graphics information loaded...");
						dfd.resolve();
					});
				});
			});

			return dfd;
		};
	};

	return SystemReport_GraphicsDeviceModel;
});
