define(["system/memory/MemoryInterface"], function(MemoryInterface) {
	/*
		Memory Model - Provides a model for Device memory information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/memory/MemoryInterface class
		which provides access to lower level memory information for the device.
	 */
	function SystemReport_MemoryModel() {
		var self = this;
		self.total_available = 1024;
		self.total_used = 0;

		//
		// Attempts to populate the model
		//
		self.generate = function() {
			var dfd = $.Deferred();
			
			// Get Total RAM Available
			logger.info("Getting Total Memory...");
			MemoryInterface.getRAMTotalAvailable().done(function(totalRAM) {
				// Update total available
				self.total_available = totalRAM;
				// Get RAM Usage
				logger.info("Getting Memory Usage...");
				MemoryInterface.getRAMUsage().done(function(ramUsage) {
					// Calculate total used
					self.total_used = Math.round(self.total_available * (ramUsage / 100));
					// Resolve Memory Model
					logger.info("Finished loading memory information");
					dfd.resolve();

				}).fail(function() {
					// Reject Memory Model
					dfd.reject();
				});
			}).fail(function() {

				// Reject Memory Model
				dfd.reject();
			});

			return dfd;
		};
	}

	return SystemReport_MemoryModel;
});
