define(["system/network/NetworkInterface"], function(NetworkInterface) {
	/*
		Network Adapter Model - Provides a model for Device network information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/network/NetworkInterface class
		which provides access to lower level networking information for the device.
	 */
	function SystemReport_NetworkAdapterModel() {
		var self = this;
		self.type = '';
		self.use_dhcp = false;
		self.ipv4_address = '';
		self.subnet_mask = '';
		self.default_gateway = '';
		self.dns1 = '';
		self.dns2 = '';
		self.mac_address = '';

		//
		// When called attempts to populate the model with informaton from the device
		//
		self.generate = function() {

			var dfd = $.Deferred();

			logger.info('Getting network information');

			try {
				// Get active network connection type
				logger.info("Getting network connection type..");
				NetworkInterface.getActiveNetworkConnectionType().done(function(activeType) {
					self.type = activeType;
				}).always(function(){
					// Get the rest of the device details. Could use $.when but using .always() instead - this seemed to work better for SSSP as each call to the SDK occurs one at a time.
				
					// DHCP enabled?
					logger.info("Getting DHCP..");
					NetworkInterface.isDHCP().done(function(isDHCP){
						self.use_dhcp = isDHCP;
					}).always(function(){

						// Get device IP
						logger.info("Getting IP..");
						NetworkInterface.getIP().done(function(ip){
							self.ipv4_address = ip;
						}).always(function() {

							// Get subnet mask
							logger.info("Getting subnet mask..");
							NetworkInterface.getSubnetMask().done(function(subnetMask) {
								self.subnet_mask = subnetMask;
							}).always(function() {

								// Get default gateway
								logger.info("Getting default gateway..");
								NetworkInterface.getDefaultGateway().done(function(defaultGateway){
									self.default_gateway = defaultGateway;
								}).always(function() {

									// Get DNS settings
									logger.info("Getting DNS..");
									NetworkInterface.getDNS().done(function(dns){
										self.dns1 = dns;
									}).always(function() {

										// Get MAC address
										logger.info("Getting MAC..");
										NetworkInterface.getMAC().done(function(mac) {
											self.mac_address = mac;
										}).always(function() {
											logger.info("Network info loaded..");
											// Resolve Generation
											dfd.resolve();
										});
									});
								});
							});
						});
					});
				});
			} catch(e) {
				logger.error('Error whilst getting network information: ' + e.message);
				dfd.reject();
			}

			return dfd;
		};
	}

	return SystemReport_NetworkAdapterModel;
});
