define(["system/storage/StorageInterface"], function(StorageInterface){
	/*
		Storage Model - Provides a model for Device storage information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/storage/StorageInterface class
		which provides access to lower level storage information for the device.
	 */
	function SystemReport_StorageDeviceModel(){
		var self = this;
		
		// Defaults
		self.name = 'Internal Storage';
		self.format = 'Unknown';
		self.total_available = 0;
		self.total_used = 0;
		self.type = 'Fixed Internal';

		//
		// Attempts to populate the storage system report model
		//
		self.generate = function() {

			var dfd = $.Deferred();

			// Get storage Name
			logger.info("Getting Storage name..");
			StorageInterface.getStorageName().done(function(storageName){
				self.name = storageName;
			}).always(function() {
				// Get Storage format
				logger.info("Getting storage format..");
				StorageInterface.getStorageFormat().done(function(storageFormat){
					self.format = storageFormat;
				}).always(function() {
					// Get total available space
					logger.info("Getting available storage..");
					StorageInterface.getTotalStorageAvailable().done(function(availableSpace) {
						self.total_available = availableSpace;
					}).always(function() {
						// Get total used
						logger.info("Getting used storage..");
						StorageInterface.getTotalStorageUsed().done(function(usedSpace){
							self.total_used = usedSpace;
						}).always(function() {
							logger.info("Getting storage type..");
							StorageInterface.getStorageType().done(function(storageType){
								self.type = storageType;
							}).always(function() {
								// Resolve storage device system report generation
								logger.info("Finished loading storage information.");
								dfd.resolve();
							});
						});
					});
				});
			});

			return dfd;
		};
	}
	
	return SystemReport_StorageDeviceModel;
});
