define(["system/DateTime", "system/SystemInterface", "controllers/UserConfigurationController"], function(DateTime, SystemInterface, UserConfigurationController) {
	/*
		System Report Model - Provides a model for System information
		-------------------------------------------------------------------------
	*/
	function SystemReport_SystemModel() {
		var self = this;

		// Defaults
		self.name = "Signagelive HTML5 Player";
		self.operating_system_name = "Unknown";
		self.operating_system_version = "Unknown";
		self.manufacturer = "Unknown";
		self.model = "Unknown";
		self.up_time = 0;
		self.system_time = DateTime.now();
		self.time_zone = UserConfigurationController.timeZone().id;
		self.utc_offset = 0;

		//
		// Attempts to populate the model with actual data using the SystemInterface
		//
		self.generate = function() {
			var dfd = $.Deferred();
			
			// Get system name
			logger.info("Getting System Name..");
			SystemInterface.getSystemName().done(function(sysName){
				self.name = sysName;
			}).always(function(){
				// Get operating system name
				logger.info("Getting IS Name..");
				SystemInterface.getOSName().done(function(osName){
					self.operating_system_name = osName;
				}).always(function() {
					// Get operating system version
					logger.info("Getting OS Version..");
					SystemInterface.getOSVersion().done(function(osVersion){
						self.operating_system_version = osVersion;
					}).always(function(){
						// Get manufacturer
						logger.info("Getting Manufacturer..");
						SystemInterface.getManufacturer().done(function(manufacturer){
							self.manufacturer = manufacturer;
						}).always(function() {
							// Get Model
							logger.info("Getting Model..");
							SystemInterface.getModel().done(function(modelName){
								self.model = modelName;
							}).always(function() {
								// Get up time
								logger.info("Getting System Up Time..");
								SystemInterface.getSystemUpTime().done(function(upTime){
									self.up_time = upTime;
								}).always(function() {
									// Get Time Zone
									logger.info("Getting Timezone..");
									SystemInterface.getTimeZone().done(function(timeZone){
										self.time_zone = timeZone;
									}).always(function() {
										// Get UTC Offset
										logger.info("Getting UTC Offset..");
										SystemInterface.getUTCOffset().done(function(utcOffset){
											self.utc_offset = utcOffset;
										}).always(function(){
											logger.info("System data loaded.");
											// Get System Time (Update because the class may have been instantiated some time ago)
											self.system_time = DateTime.now();
											// Resolve System Info Report Generation
											dfd.resolve();
										});
									});
								});
							});
						});
					});
				});
			});

			return dfd;
		};
	}

	return SystemReport_SystemModel;
});

