define(["system/temperature/TemperatureInterface"], function(TemperatureInterface) {
	/*
		Temperature Model - Provides a model for Device temperature information
		-------------------------------------------------------------------------
		This model generates all of its information from the system/temperature/TemperatureInterface class
		which provides access to lower level temperature information for the device.
	 */
	function SystemReport_TemperatureModel() {
		var self = this;

		// Defaults
		self.sensor_name = 'Screen';
		self.temperature = 0;
		self.temperature_unit = "celsius";

		//
		// Attempts to populate the temperature model
		//
		self.generate = function() {
			var dfd = $.Deferred();

			// Get Temperature
			logger.info("Getting Temperature..");
			TemperatureInterface.getTemperature().done(function(temp) {
				self.temperature = temp;
			}).always(function() {
				// Get Temperature Unit
				logger.info("Getting Temperature unit..");
				TemperatureInterface.getTemperatureUnit().done(function(unit) {
					self.temperature_unit = unit;
				}).always(function(){
					// Get Sensor Name
					logger.info("Getting sensor name..");
					TemperatureInterface.getSensorName().done(function(sensorName){
						self.sensor_name = sensorName;
					}).always(function(){
						// Resolve Temperature Model
						logger.info("Finished loading temperature information");
						dfd.resolve();
					});
				});
			});

			return dfd;
		};
	}

	return SystemReport_TemperatureModel;
});
