define(["properties/App.Settings", "system/io/File", "models/PusherSubscriptionModel"], function(Settings, File, PusherSubscriptionModel) {
    /**
     * [UserConfigurationModel description]
     */
    function UserConfigurationModel() {
        var self = this;

        self.isMuted = true;
        self.showTrace = false;
        self.rotation = {
            displayName: 'No rotation',
            angle: 0,
            id: 0
        };
        self.layoutFit = {
			displayName : 'Fill - Resize layouts to fill display area (Default)',
			id : 'fill'
		};
        self.rotateVideos = false;
        self.useNTP = false;
        self.usePMModeForScreenManagement = false; //NOTE: Or should it link to the Settings.user.mySettings.usePMModeForScreenManagement
        self.timeZone = {
            displayName: '(GMT) Greenwich Mean Time : Dublin, Edinburgh, Lisbon, London',
            id: 'Europe/London'
        };
        self.interactiveKiosk = {
            enabled: false,
            options: {
                idleIntervalInSeconds: 15,
                trigger: {
                    key: "t",
                    type: "KeyboardEvent"
                }
            }
        };
        self.modules = {
            webBasedInterrupts: {
                enabled: false,
                options: {
                    pusherSettings: null,
                    trigger: null
                }
            },
            interactiveKiosk: {
                enabled: false,
                options: {
                    idleIntervalInSeconds: 15,
                    trigger: {
                        key: "t",
                        type: "KeyboardEvent"
                    }
                }
            },
            GPIOBasedInterrupts: {
                enabled: false,
                options: {
                    trigger: {
                        type: "KeyboardEvent"
                    }
                }
            },
            RFIDBasedInterrupts: {
                enabled: false,
                options: {}
            }
        };

        self.remoteFirmwareUpdates = {
            enabled: false,
            configured: false
        };

        self.settings = {};
        self.hasSaved = false;

        /**
         * Initialize
         * @return {[type]} [description]
         */
        self.initialize = function() {
            var dfd = $.Deferred();

            // load the data
            try {
                logger.debug('User Configuration Model: Loading user settings data');
                File.read(Settings.application.fileSystem.systemFiles.settingsFilePath).done(function(settingsData) {
                    if (settingsData != null) {
                        logger.info('Reading settings: ' + settingsData);
                        self.settings = JSON.parse(settingsData);
                        if (self.settings.showTrace != undefined)
                            self.showTrace = self.settings.showTrace;

                        if (self.settings.isMuted != undefined)
                            self.isMuted = self.settings.isMuted;

                        if (self.settings.rotation != undefined)
                            self.rotation = self.settings.rotation;

                        if (self.settings.rotateVideos != undefined)
                            self.rotateVideos = self.settings.rotateVideos;

                        if (self.settings.timeZone != undefined)
                            self.timeZone = self.settings.timeZone;
                        
                        if (self.settings.layoutFit != undefined)
                            self.layoutFit = self.settings.layoutFit;

                        if (self.settings.useNTP != undefined)
                            self.useNTP = self.settings.useNTP;

                        if (self.settings.usePMModeForScreenManagement != undefined)
                            self.usePMModeForScreenManagement = self.settings.usePMModeForScreenManagement;

                        if (self.settings.modules != undefined) {
                            self.modules = self.settings.modules;
                            $.extend(true, Settings.application.modules, self.modules);
                        }

                        if (self.settings.remoteFirmwareUpdates != undefined) {
                            self.remoteFirmwareUpdates = self.settings.remoteFirmwareUpdates;
                            $.extend(true, Settings.application.remoteFirmwareUpdates, self.remoteFirmwareUpdates);
                        }

                        self.hasSaved = true;
                    }
                    dfd.resolve(self);
                }).fail(function() {
                    logger.error('User Configuration Model: Could not read settings file. Does it exist?');
                    dfd.resolve(self);
                });

            } catch (e) {
                logger.error('User Configuration Model: Error reading user settings ' + e.message);
                dfd.reject();
            }

            return dfd;
        };

        /**
         * [save description]
         * @return {[type]} [description]
         */
        self.save = function() {
            var dfd = $.Deferred();

            try {
                logger.debug('User Configuration Mode: Save User Config');

                // save the settings file to indicate that setup has been completed.
                // TODO iterate the settings to generate the serialized object
                self.settings.isMuted = self.isMuted;
                self.settings.showTrace = self.showTrace;
                self.settings.rotation = self.rotation;
                self.settings.timeZone = self.timeZone;
                self.settings.layoutFit = self.layoutFit;
                self.settings.useNTP = self.useNTP;
                self.settings.usePMModeForScreenManagement = self.usePMModeForScreenManagement;
                self.settings.modules = self.modules;
                self.settings.remoteFirmwareUpdates = self.remoteFirmwareUpdates;
                self.settings.rotateVideos = self.rotateVideos;

                $.extend(true, Settings.application.modules, self.modules);
                $.extend(true, Settings.application.remoteFirmwareUpdates, self.remoteFirmwareUpdates);
                //$.extend(true, Settings.application.rotateVideos, self.rotateVideos);

                File.write(Settings.application.fileSystem.systemFiles.settingsFilePath, JSON.stringify(self.settings)).done(function() {
                    logger.info('Saving settings: ' + JSON.stringify(self.settings, undefined, 2));
                    self.hasSaved = true;
                    dfd.resolve(self);
                }).fail(function() {
                    dfd.reject();
                });

            } catch (e) {
                logger.error('User Configuration Model: Error whilst saving user settings ' + e.message);
                dfd.reject();
            }

            return dfd;
        };
    }

    return UserConfigurationModel;
});