define(["system/DateTime"], function(DateTime) {
	function ValidityModel(data) {
		var self = this;
		if (data.start != null)
			self.start = DateTime.parse(data.start);

		if (data.end != null)
			self.end = DateTime.parse(data.end);

		self.startTime = data.start_time;
		self.endTime = data.end_time;

		self.days = data.days;

		self.isCurrentlyValid = function(date) {

			if(!date)
				date = DateTime.now();

			var dateRangeValid = isDateRangeValid(date);
			var timeRangeValid = isTimeRangeValid(date);
			var dayValid = isDayValid(date);
			var valid = (dateRangeValid && timeRangeValid && dayValid);

			return valid;
		}
		function isDateRangeValid(date) {
			var valid = false;
			if (self.start != null & self.end != null) {
				if (date >= self.start && date < self.end) {
					valid = true;
				}
			} else {
				valid = true;
			}

			return valid;
		}

		function isTimeRangeValid(date) {
			var valid = false;

			if (self.startTime != null & self.endTime != null) {
				var currentHour = (date.getHours() < 10) ? ("0" + date.getHours()) : date.getHours();
				var currentMinute = (date.getMinutes() < 10) ? ("0" + date.getMinutes()) : date.getMinutes();
				var currentSecond = (date.getSeconds() < 10) ? ("0" + date.getSeconds()) : date.getSeconds();

				var dateTime = currentHour + ":" + currentMinute + ":" + currentSecond; //get current date time information - HH:mm:ss only
				
				if(self.endTime < self.startTime) {
					if(dateTime >= self.startTime && dateTime <= "23:59:59" || dateTime >= "00:00:00" && dateTime < self.endTime) { //if startTime is after current dateTime and its before midnight, or if dateTime is after midnight and its before the scheduled endTime
						valid = true;
					}
				}

				if (dateTime >= self.startTime && dateTime < self.endTime) { //else check if dateTime is after the scheduled (same-day) startTime yet before the endTime
					valid = true;
				}

			} else {
				valid = true;
			}
			return valid;
		}

		function isDayValid(date) {
			if (self.days == null)
				return true;
				
			var valid = false;			

			var dayNumber = date.getDay();
			switch(dayNumber) {
				case 0:
					// Sunday
					valid = ((self.days & 1) == 1);
					break;
				case 1:
					// Monday
					valid = ((self.days & 2) == 2);
					break;
				case 2:
					// Tuesday
					valid = ((self.days & 4) == 4);
					break;
				case 3:
					// Wednesday
					valid = ((self.days & 8) == 8);
					break;
				case 4:
					// Thursday
					valid = ((self.days & 16) == 16);
					break;
				case 5:
					// Friday
					valid = ((self.days & 32) == 32);
					break;
				case 6:
					// Saturday
					valid = ((self.days & 64) == 64);
					break;
			}

			return valid;
		}

	}

	return ValidityModel;
});
