define(["properties/App.Settings", 'system/network/NetworkInterface', 'system/io/File'], function(Settings, NetworkInterface, File) {
	function ClientIDGenerator() {
		var self = this;
		self.loadClientID = function() {
            var dfd = $.Deferred();
            logger.debug("Loading Client ID....");
            // load the client Id
            File.read(Settings.application.fileSystem.systemFiles.clientIdFilePath).done(function(data) {
                if (data != null && data != "") {
                    var clientIdData = JSON.parse(data);

                    if (Settings.application.features.validateClientId === true)
                    {
                        // Validate client Id
                        self.validateClientID(clientIdData.clientId).done(function(err, validated) {
                            if (err || validated === false)
                            {
                                logger.error('CRITICAL: Mac validation failed');
                                var retryOnFailure = Settings.application.features.dynamicallyCalculateClientId;
                                self.generateClientID(retryOnFailure).done(function(id) {
                                    logger.debug("New Client ID Generated: " + id);
                                    Settings.application.api.clientId = id;
                                    if (!Settings.application.features.dynamicallyCalculateClientId) {
                                        File.write(Settings.application.fileSystem.systemFiles.clientIdFilePath, JSON.stringify({
                                            clientId: Settings.application.api.clientId
                                        })).done(function() {
                                            logger.debug("New Client ID Stored.");
                                            return dfd.resolve();
                                        });
                                    } else {
                                        return dfd.resolve();
                                    }
                                });
                            }
                            logger.debug('Mac validation passed');
                            logger.info('Stored Client ID is ' + clientIdData.clientId);
                            Settings.application.api.clientId = clientIdData.clientId;
                            dfd.resolve();
                        });
                    }
                    else
                    {
                        // Validation not implemented - trust the stored client id
                        logger.info('Stored Client ID is ' + clientIdData.clientId);
                        Settings.application.api.clientId = clientIdData.clientId;
                        dfd.resolve();
                    }
                } else {
                    var retryOnFailure = Settings.application.features.dynamicallyCalculateClientId;
                    self.generateClientID(retryOnFailure).done(function(id) {
                        logger.debug("New Client ID Generated: " + id);
                        Settings.application.api.clientId = id;
                        if (!Settings.application.features.dynamicallyCalculateClientId) {
                            File.write(Settings.application.fileSystem.systemFiles.clientIdFilePath, JSON.stringify({
                                clientId: Settings.application.api.clientId
                            })).done(function() {
                                logger.debug("New Client ID Stored.");
                                dfd.resolve();
                            });
                        } else {
                            dfd.resolve();
                        }
                    });
                }
            }).fail(function() {
                var retryOnFailure = Settings.application.features.dynamicallyCalculateClientId;
                self.generateClientID(retryOnFailure).done(function(id) {
                    logger.debug("New Client ID Generated: " + id);
                    Settings.application.api.clientId = id;
                    if (!Settings.application.features.dynamicallyCalculateClientId) {
                        File.write(Settings.application.fileSystem.systemFiles.clientIdFilePath, JSON.stringify({
                            clientId: Settings.application.api.clientId
                        })).done(function() {
                            logger.debug("New Client ID Stored.");
                            dfd.resolve();
                        });
                    } else {
                        dfd.resolve();
                    }
                });
            });
            return dfd;
		};
		
		self.generateClientID = function() {
			var dfd = $.Deferred();
			NetworkInterface.getMAC().done(
				function(mac) {
					logger.debug("MAC Address: " + mac);
					dfd.resolve(mac.replace("/:/g", "-"));
				}
			)
			.fail(
				function() {
					dfd.reject();
				}
			);
			return dfd;
        };
        
        self.validateClientID = function(clientId)
        {
            var dfd = $.Deferred();
            NetworkInterface.validateStoredMAC(clientId).done(function(err, validated) {
                dfd.resolve(err, validated);
            });
            return dfd;
        };
	}

	return ClientIDGenerator;
});

