define(["models/LocalFileList","properties/App.Settings"], function(LocalFileList, Settings) {
    /*
    	DownloadPlugin - Handles Downloading of files to the physical file system.
    	----------------------------------------------------------------------------
    	Uses LG SDK Storage().copyFile() method.

    	KNOWN ISSUE: This method fails when presented with a HTTP redirect. This has been most widely seen
    	with RSS tickers on the live environment. I have fixed this temporarily using string replacement
    	to change login.signagelive.com/legacy/getrss to sl.signagelive.com/legacy/getrss
     */
    function DownloadPlugin() {
        var self = this;
        var _remoteUrl;
        var _localUrl;
        var _Deferred;

        self.downloadFile = function(remoteUrl, localUrl) {
            _remoteUrl = remoteUrl;
            _localUrl = localUrl;
            // Temporary fix for RSS tickers
            _remoteUrl = _remoteUrl.replace("login.signagelive.com/getrss.asp", "sl.signagelive.com/Legacy/getrss.asp");
            _remoteUrl = _remoteUrl.replace("test.signagelive.com/getrss.asp", "test-sl.signagelive.com/Legacy/getrss.asp");
            // Update URLS for API segregation if TLS 1.2 is not supported
            // If use Legacy TLS URL is true then we need to replace the default app urls with the legacy tls urls
            if (Settings.application.features.supportsTLS12 === false && Settings.application.api.useLegacyTlsUrl === true) {
                _remoteUrl = _remoteUrl.replace(Settings.application.api.url, Settings.application.api.legacyTlsUrl);
                _remoteUrl = _remoteUrl.replace(Settings.application.urls.m2mUrl, Settings.application.urls.legacyTlsM2mUrl);
                _remoteUrl = _remoteUrl.replace(Settings.application.urls.contentUrl, Settings.application.urls.legacyTlsContentUrl);
                _remoteUrl = _remoteUrl.replace(Settings.application.urls.rssProxyUrl, Settings.application.urls.legacyTlsRssProxyUrl);
            }

            logger.info('File Download Plugin: Start download from: ' + _remoteUrl + ' to: ' + _localUrl);

            _Deferred = $.Deferred();

            // Download file using LG SDK copyFile method
            new Storage().copyFile(
                // Download Succeeded
                function(result) {
                    LocalFileList.add(_localUrl).always(function() {
                        logger.debug('File Download Plugin: Downloaded New File: ' + _localUrl);
                        _Deferred.resolve(_remoteUrl, _localUrl);
                    });
                },
                // Download Failed
                function(result) {
                    logger.error('File Download Plugin: Error whilst downloading file from: ' + _remoteUrl + '. Error: [' + result.errorCode + '] ' + result.errorText);
                    _Deferred.reject(_remoteUrl, _localUrl);
                },
                // Copy FilesOptions
                {
                    source: _remoteUrl,
                    destination: _localUrl
                }
            );

            return _Deferred.promise();
        };
    }

    return DownloadPlugin;
});