define(["gui/views/SVGConstants"], function(userConfig) {
	function VideoPlugin(parentId, x, y, width, height, callbacks) {
		var self = this;

		var _muted = true;
		var _isPlaying = false;

		// need to scale the Media Window as its resolution is 960x540!
		var xScalar = 960 / document.documentElement.clientWidth;
		var yScalar = 540 / document.documentElement.clientHeight;

		self.x = x * xScalar;
		self.y = y * yScalar;
		self.width = width * xScalar;
		self.height = height * yScalar;

		var _callbacks = callbacks;

		self.videoPlugin = document.getElementById("videoPlugin");

		function stopAndFail() {
			_isPlaying = false;

			self.videoPlugin.Stop();

			if (_callbacks.fail != null)
				_callbacks.fail();
		}

		function stopAndComplete() {
			_isPlaying = false;

			self.videoPlugin.Stop();

			if (_callbacks.complete != null)
				_callbacks.complete();
		}


		self.audioPlugin = document.getElementById("audioPlugin");
		self.tvmwPlugin = document.getElementById("TVMWplugin");

		self.initComplete = false;

		self.play = function(url) {
			try {
				// is it requied to set the callbacks each time to the callbacks on 'this' instance
				// as the player and all zones use the same instance of the native video control
				setUpVideoControlCallbacks();

				self.originalSource = self.tvmwPlugin.GetSource();

				if (self.originalSource != 43) {
					self.tvmwPlugin.SetSource(43);

					setTimeout(function() {

						startVideo(url);

					}, 1000);
				} else {
					startVideo(url);
				}

			} catch(e) {
				logger.error('Error whilst playing video ' + e.message);
			}
		};
		function startVideo(url) {
			// setup the player with the url
			self.videoPlugin.InitPlayer(url);

			// set buffer
			self.videoPlugin.SetInitialBuffer(1024);

			// set the display area of the video
			self.videoPlugin.SetDisplayArea(self.x, self.y, self.width, self.height);

			self.videoPlugin.StartPlayback();

			self.setMute(_muted);

			_isPlaying = true;
		}


		self.stop = function() {
			try {
				if (_isPlaying) {

					logger.debug('Video stop...');
					self.videoPlugin.Stop();

					_isPlaying = false;
				}
			} catch(e) {
				logger.error('Error whilst stopping video ' + e.message);
			}
		};

		self.setMute = function(muted) {
			logger.debug('Video Plugin set mute');

			//TODO:do properly
			return;

			_muted = muted;

			// mute/unmute the video
			var muteState = (_muted) ? 1 : 0;
			self.audioPlugin.SetUserMute(muteState);

		};

		self.volumeUp = function() {
			if (_isPlaying) {
				self.setMute(false);
				self.audioPlugin.SetVolumeWithKey(0);
			}
		};

		self.volumeDown = function() {
			if (_isPlaying) {
				self.setMute(false);
				self.audioPlugin.SetVolumeWithKey(1);
			}
		};
		function setUpVideoControlCallbacks() {
			self.videoPlugin.OnRenderingComplete = function() {
				logger.debug("Video Complete Callback");

				stopAndComplete();
			};

			self.videoPlugin.OnRenderError = function(type) {
				switch(type) {
					case 1:
						logger.error('Error whilst playing video: Unsupported Container');
						break;
					case 2:
						logger.error('Error whilst playing video: Unsupported Video Codec');
						break;
					case 3:
						logger.error('Error whilst playing video: Unsupported Audio Codec');
						break;
					case 4:
						logger.error('Unsupported Video: Resolution');
						break;
					default:
						logger.error('Error whilst playing video: Unknown Rendering Error ' + type);
				}

				stopAndFail();
			};

			self.videoPlugin.OnStreamNotFound = function() {
				logger.error('Error whilst playing video: Stream Not Found');
				stopAndFail();
			};

			self.videoPlugin.OnNetworkDisconnected = function() {
				logger.error('Error whilst playing video: Network Disconnected');

				stopAndFail();
			};

			self.videoPlugin.OnAuthenticationFailed = function() {
				logger.error('Error whilst playing video: Authentication Failed');

				stopAndFail();
			};

			self.videoPlugin.OnConnectionFailed = function() {
				logger.error('Error whilst playing video: Connection Failed');

				stopAndFail();
			};
		}

	}

	return VideoPlugin;
});
