define(["properties/App.Settings"], function(Settings) {
    /*
    	RemoteControlInterface provides access to low level remote control Information
     */
    function RemoteControlInterface() {

        var self = this;

        // Supported LG Key Codes
        self.keys = {
            KEY_INFO: 405, // NOTE: This is actually the Yellow Key on the LG Remote as it does not have an Info Key
            KEY_SETUP: 406, // NOTE: This is actually the blue key on the remote. This should be changed to
            KEY_PLAY: 415,
            KEY_PAUSE: 19,
            KEY_RW: 412,
            KEY_FF: 417,
            KEY_ENTER: 13,
            KEY_UP: 38,
            KEY_DOWN: 40,
            KEY_LEFT: 37,
            KEY_RIGHT: 39,
            KEY_0: 48,
            KEY_1: 49,
            KEY_2: 50,
            KEY_3: 51,
            KEY_4: 52,
            KEY_5: 53,
            KEY_6: 54,
            KEY_7: 55,
            KEY_8: 56,
            KEY_9: 57,
            KEY_VOL_UP: -1, // Not Implemented
            KEY_VOL_DOWN: -1, // Not Implemented
            KEY_MUTE: -1, // Not Implemented - Note there is a mute button but it is not detected in the application
            KEY_POWER: 409 // Signage.KeyCode.POWER unavailable on init, DISC_POWER_OFF is Monitor Off and 706, DISC_POWER_ON is Monitor On and 705
        };

        // Used to translate the keyCodes to the interrupt key value returned in the interrupt configuration
        self.keyCodesToInterruptKeys = {
            48: 'D0',
            49: 'D1',
            50: 'D2',
            51: 'D3',
            52: 'D4',
            53: 'D5',
            54: 'D6',
            55: 'D7',
            56: 'D8',
            57: 'D9'
        };

        /**
         * Perform any device-specific required initialization here
         * Add the ability to intercept the power command for webOS 4.1+
         */
        self.initialize = function () {
            var dfd = $.Deferred();
            try {
                if (Settings.application.webOSVersion >= 4.1) {
                    new Signage().addKeyItem(function () { //only supported on webOS 4.1+
                            // Success
                            logger.info("Remote Control Interface: addKeyItem success");
                            dfd.resolve();
                        },
                        function (result) {
                            // Failed
                            logger.error("Remote Control Interface: Error adding key item: Error [" + result.errorCode + "] " + result.errorText);
                            dfd.resolve(); //resolve anyway
                        }, {
                            keyCode: 0x00000000,
                            virtualKeyCode: self.keys.KEY_POWER, //this key code may be different depending on device model
                            attribute: 2
                        });
                }
            } catch (e) {
                dfd.resolve();
            }

            return dfd;
        }

        //
        //	Not Implemented - Not supported in LG SDK
        //
        self.getRemoteControlStatus = function() {

            var dfd = $.Deferred();

            setTimeout(function() {
                //logger.warning("Remote Control Interface: Error: Get Remote Control Status Not Implemented.");
                dfd.reject();
            }, 100);

            return dfd;
        };

        //
        //	Not Implemented - Not Supported in LG SDK
        //
        self.getButtonLockStatus = function() {

            var dfd = $.Deferred();

            setTimeout(function() {
                //logger.warning("Remote Control Interface: Error: Get Button Lock Not Implemented.");
                dfd.reject();
            }, 100);

            return dfd;
        };

        //
        //	Not Implemented - Not Supported in LG SDK
        //
        self.getSafetyLockStatus = function() {

            var dfd = $.Deferred();

            setTimeout(function() {
                //logger.warning("Remote Control Interface: Error: Get Safety Lock Status Not Implemented.");
                dfd.reject();
            }, 100);

            return dfd;
        };

    }

    // The object is a singleton as there is only a single plugin added to the application
    return new RemoteControlInterface();
});