define(["models/IdleState"], function(IdleState) {
	/*
		Idle Notification Interface provides a generic interface for notifying the application
		when the idle state changes.

		This implementation is heavily influenced by the Chrome.Idle API but should provide us the flexibility
		to implement this in other platforms if and when it becomes possible to do so.
	 */
	function IdleNotificationInterface() {
		var self = this;

		self.onStateChange = {};

		/**
		 * Set the idle interval term used to determine the period of
		 * inactivity required for system to be deemed inactive
		 * @param {[type]} idleIntervalInSeconds [description]
		 */
		self.setIdleInterval = function(idleIntervalInSeconds)
		{
			var dfd = $.Deferred();

			setTimeout(function(){
				logger.warning("IdleNotificationInterface: setIdleInterval called but not implemented");
				dfd.reject();
			}, 150);
			
			return dfd;
		};

		/**
		 * Get the current Idle Status
		 * @param  {[type]} idleIntervalInSeconds [description]
		 * @return {[type]}                       [description]
		 */
		self.queryState = function(idleIntervalInSeconds)
		{
			var dfd = $.Deferred();
			
			setTimeout(function(){
				logger.warning("IdleNotificationInterface: onStateChange.queryState called but not implemented");
				dfd.reject();
			}, 150);

			return dfd;
		};

		/**
		 * Register callback that is fired when the idle status changes
		 * @param {Function} callback [description]
		 * @return Model/IdleState enum
		 */
		self.onStateChange.addListener = function(callback)
		{
			var dfd = $.Deferred();

			setTimeout(function(){
				logger.warning("IdleNotificationInterface: onStateChange.addListener called but not implemented");
				dfd.reject();
			}, 150);

			return dfd;
		};

	}
	
	// The object is a singleton as there is only a single inteface added to the application
	return new IdleNotificationInterface();
});