define(["system/sensors/LunaAPIInterface"], function(LunaAPIInterface) {
	
	/**
	 * NFC Interface
	 * ---------------------------------
	 * Allows LG client to turn on/off nfc and write to the nfc tag
	 */
	function NFCInterface() {

		var self = this;
		_enabled = false;
		
		/**
		 * [writeTag description]
		 * @param  {[type]} data [description]
		 * @return {[type]}      [description]
		 */
		self.writeTag = function(data)
		{
			var dfd = $.Deferred();

			logger.debug("NFC Interface: Updating NFC Tag" + data);

			// Call Luna API
			var url = "luna://com.webos.service.nfc/tag/writeTag";
			var params = JSON.stringify({ "url" : data });
			LunaAPIInterface.call(
				url,
				params,
				// Callback
				function(response){
					if (response.returnValue === true)
					{
						logger.debug("Successfully updated NFC tag: " + data);
						dfd.resolve();
					}
					else
					{
						logger.error("Error updating NFC tag");
						dfd.reject();
					}
				}
			);

			return dfd;
		};
		
		/**
		 * [enable description]
		 * @return {[type]} [description]
		 */
		self.enable = function()
		{
			var dfd = $.Deferred();

			logger.debug("NFC Interface: Enabling NFC");

			// Call Luna API
			var url = "luna://com.webos.service.nfc/adapter/setState";
			var params = JSON.stringify({ "powered" : true });
			LunaAPIInterface.call(
				url,
				params,
				// Callback
				function(response){
					if (response.returnValue === true)
					{
						logger.debug("Successfully enabled NFC");
						_enabled = true;
						dfd.resolve();
					}
					else
					{
						logger.error("Error enabling NFC");
						dfd.reject();
					}
				}
			);

			return dfd;	
		};

		/**
		 * [disable description]
		 * @return {[type]} [description]
		 */
		self.disable = function()
		{
			var dfd = $.Deferred();

			logger.debug("NFC Interface: Disabling NFC");

			// Call Luna API
			var url = "luna://com.webos.service.nfc/adapter/setState";
			var params = JSON.stringify({ "powered" : false });
			LunaAPIInterface.call(
				url,
				params,
				// Callback
				function(response){
					if (response.returnValue === true)
					{
						logger.debug("Successfully disabled NFC");
						_enabled = false;
						dfd.resolve();
					}
					else
					{
						logger.error("Error disabling NFC");
						dfd.reject();
					}
				}
			);

			return dfd;
		};

		/**
		 * [isEnable description]
		 * @return {[type]} [description]
		 */
		self.isEnabled = function()
		{
			var dfd = $.Deferred();

			setTimeout(function(){
				dfd.resolve(_enabled);
			}, 50);

			return dfd;	
		};

	}
	
	// The object is a singleton as there is only a single inteface added to the application
	return new NFCInterface();
});