define(function() {
	/*
		Sound Interface - Access to the LG SDK methods related to sound and volume control / information
	*/
	function SoundInterface() {
	
		var self = this;

		//
		// Get volume level of LG Panels
		//
		self.getVolume = function() {
			var dfd = $.Deferred();

			new Sound().getSoundStatus(
		        function(result) {
		            // Success
		            logger.info("Sound Interface: Volume level: " + result.level);
		            dfd.resolve(result.level);
		        },
		        function(result) {
		            // failure
		            logger.error("Sound Interface: Error getting volume level: Error [" + result.errorCode + "] " + result.errorText);
		            dfd.reject();
		        }  
    		);

			return dfd;
		};

		//
		// Set the volume level on LG Panels
		//
		self.setVolume = function(level) {
			var dfd = $.Deferred();

			var options = {};
			options.level = level;

			new Sound().getSoundStatus(
		        function(result) {
		            // Success
		            logger.info("Sound Interface: Volume Level changed to: " + level);
		            dfd.resolve(result.level);
		        },
		        function(result) {
		            // failure
		            logger.error("Sound Interface: Error changing volume level: Error [" + result.errorCode + "] " + result.errorText);
		            dfd.reject();
		        },
		        options
    		);
			

			return dfd;
		};

		//
		// Get Mute Status on LG Panels
		//
		self.getMuteStatus = function() {
			var dfd = $.Deferred();

			new Sound().getSoundStatus(
		        function(result) {
		            // Success
		            logger.info("Sound Interface: Mute status: " + result.muted);
		            dfd.resolve(result.muted);
		        },
		        function(result) {
		            // failure
		            logger.error("Sound Interface: Error getting mute status: Error [" + result.errorCode + "] " + result.errorText);
		            dfd.reject();
		        }  
		    );

			return dfd;
		};

		//
		// Set Mute Status on LG Panels
		//
		self.setMuteStatus = function(muted) {
			var dfd = $.Deferred();

			var options = {};
		    options.muted = muted;
		     
		    new Sound().getSoundStatus(
		        function(result) {
		            // Success
		            logger.info("Sound Interface: Set mute status to: " + muted);
		            dfd.resolve();
		        },
		        function(result) {
		            // failure
		            logger.error("Sound Interface: Error setting mute status: Error [" + result.errorCode + "] " + result.errorText);
		            dfd.reject();
		        },
		        options 
		    );

			return dfd;
		};
	}

	return new SoundInterface();
});
