define(function() {
	/*
		StorageInterface provides access to low level storage Information
	 */
	function StorageInterface() {

		var self = this;

		//
		//	Returns the storage name
		//
		self.getStorageName = function() {

			var dfd = $.Deferred();
     
		    setTimeout(function() {
		     	var storageName = "Internal Storage";
		    	logger.info("Storage Interface: Storage Name: " + storageName);
		    	dfd.resolve(storageName);
		    }, 100);

		    return dfd;
		};


		//
		//	Returns the storage format
		//
		self.getStorageFormat = function() {

			var dfd = $.Deferred();
     
		    setTimeout(function() {
		     	var storageFormat = "";
		    	logger.info("Storage Interface: Storage Format: " + storageFormat);
		    	dfd.resolve(storageFormat);
		    }, 100);

		    return dfd;
		};

		//
		//	Not Implemented - Not Supported in generic app
		//
		self.getStorageType = function() {

			var dfd = $.Deferred();
     
		    setTimeout(function(){
		    	logger.warning("Storage Interface: Error: Get Storage Type Not Implemented.");
		    	dfd.reject();
		    }, 100);

		    return dfd;
		};

		//
		//	Returns the total storage available
		//
		self.getTotalStorageAvailable = function() {

			var dfd = $.Deferred();
     		
     		// Get storage info and on success return total available
		    getStorageInfo()
		    .done(function(info) {
		    	dfd.resolve(info.total * 0.001);
		    })
		    .fail(function(){
		    	dfd.reject();
		    });

		    return dfd;
		};

		//
		//	Returns the total storage used
		//
		self.getTotalStorageUsed = function() {

			var dfd = $.Deferred();
     
		    // Get storage info and on success return used
		    getStorageInfo()
		    .done(function(info) {
		    	dfd.resolve(info.used * 0.001);
		    })
		    .fail(function(){
		    	dfd.reject();
		    });

		    return dfd;
		};

		//
		// Internal method that calls LG SDK Storage.getStorageInfo() method
		//
		function getStorageInfo() {

			var dfd = $.Deferred();

			new Storage().getStorageInfo(
				function(result) {
					// success
					logger.info("Storage Interface: Storage Information Loaded");
					dfd.resolve(result);
				},
				function(result) {
					// failed
					logger.error("Storage Interface: Error getting storage information. Error [" + result.errorCode + "] " + result.errorText);
					dfd.reject();
				}
			);


			return dfd;

		}

	}
	
	// The object is a singleton as there is only a single plugin added to the application
	return new StorageInterface();
});
