define(["system/DateTime", "properties/App.Settings", "system/io/File", "core/SLPlayerDataAPIInterface"], function(DateTime, Settings, File, SLPlayerDataAPIInterface) {
    var proofOfPlayLogger = {
        config: null,
        traceCounter: 0,
        traceCounterMax: 50,
        totalTrace: 0,
        apiInterface: null,
        init: function(configModel, apiCreds) {
            logger.info('Creating Proof of Play Logger, API Credentials: ' + JSON.stringify(apiCreds));
            this.config = configModel;
            if (this.config != null && this.config.uploadurl != null && this.config.enabled == true) {
                apiInterface = new SLPlayerDataAPIInterface(this.config);
                setTimeout(function() {
                    logger.info('Starting proof of play API interface in 45 seconds');
                    apiInterface.initialize();
                }, 45000);     
                logger.info('Proof of Play Logger created');
            } else {
                logger.info('Proof of Play Logger configuration not found or Proof of play not enabled');
            }
        },
        updateConfig: function(configModel, apiCreds) {
            logger.debug('Updating Proof of Play Configuration');

            var configChanged = false;

            if (this.config != null) {
                if (this.config.enabled != configModel.enabled) {
                    configChanged = true;
                }

                if (this.config.uploadfrequency != configModel.uploadfrequency) {
                    configChanged = true;
                }

                if (this.config.uploadurl = !configModel.uploadurl) {
                    configChanged = true;
                }

                if (this.config.maxlogfilesizeinkb != configModel.maxlogfilesizeinkb) {
                    configChanged = true;
                }

                if (this.config.mindiskspaceinkb != configModel.mindiskspaceinkb) {
                    configChanged = true;
                }

                if (configChanged) {
                    this.config = configModel;
                    logger.debug('Proof of Play config Updated: ' + JSON.stringify(this.config));

                    if (this.config.uploadurl != null && apiCreds != null && this.config.enabled == true) {
                        Settings.application.playerdataapi.url = this.config.uploadurl;
                        logger.debug('Proof of Play - re initialising Player Data API Interface');
                        apiInterface = new SLPlayerDataAPIInterface(this.config);
                        apiInterface.initialize();
                    }
                } else {
                    logger.debug('Proof of Play config not changed: ' + JSON.stringify(this.config));
                }
            } else {
                this.config = configModel;
                logger.debug('Proof of Play config was null, now Updated: ' + JSON.stringify(this.config));
            }
        },
        debug: function(layout, mediaWindow, playlist, playlistMediaAsset, mediaAsset, start, end, playReason, endReason) {
            //logger.debug("PoP: Proof of Play Enabled: " + this.config.enabled);

            if (this.config != null && this.config.enabled) {
                // build the proof of play log entry 
                // Media Asset ID, Layout ID, Media Window ID, Playlist ID, Playlist Media Asset ID, Start Time, End Time, Play Reason, End Reason
                logString = "";

                /* All used for debugging
                 debugIDString = "PoP: ";
                debugIncludeString = "PoP: ";
				
                if (mediaAsset != null) {
                		debugIDString = "Media Asset: " + mediaAsset.externalId + ",";
                		debugIncludeString = "Media Asset: " + mediaAsset.includeInProofOfPlay + ",";
                	} else {
                		debugIDString = ",";
                		debugIncludeString = ",";
                	}
                	
                	if (layout != null) {
                		debugIDString = debugIDString + " Layout: " + layout.externalId + ",";
                		debugIncludeString = debugIncludeString + " Layout: " + layout.includeInProofOfPlay + ",";
                	} else {
                		debugIDString = debugIDString + ",";
                		debugIncludeString = debugIncludeString + ",";
                	}
                	
                	if (mediaWindow != null) {
                		debugIDString = debugIDString + " Media Window: " + mediaWindow.externalId + ",";
                		debugIncludeString = debugIncludeString + " Media Window: " + mediaWindow.includeInProofOfPlay + ",";
                	} else {
                		debugIDString = debugIDString + ",";
                		debugIncludeString = debugIncludeString + ",";
                	}
                	
                	if (playlist != null) {
                		debugIDString = debugIDString + " Playlist: " + playlist.externalId + ",";
                		debugIncludeString = debugIncludeString + " Playlist: " + playlist.includeInProofOfPlay + ",";
                	} else {
                		debugIDString = debugIDString + ",";
                		debugIncludeString = debugIncludeString + ",";
                	}
                	
                	if (playlistMediaAsset != null) {
                		debugIDString = debugIDString + " Playlist Media Asset: " + playlistMediaAsset.externalId;
                		debugIncludeString = debugIncludeString + " Playlist Media Asset: " + playlistMediaAsset.includeInProofOfPlay;
                	}
                	
                logger.debug(debugIDString);
                logger.debug(debugIncludeString);*/

                // first check if any of the objects are marked as log for proof of play
                includeInPoP = false;
                if (playlistMediaAsset != null && playlistMediaAsset.includeInProofOfPlay) {
                    includeInPoP = true;
                } else if (playlist != null && playlist.includeInProofOfPlay) {
                    includeInPoP = true;
                } else if (mediaWindow != null && mediaWindow.includeInProofOfPlay) {
                    includeInPoP = true;
                } else if (layout != null && layout.includeInProofOfPlay) {
                    includeInPoP = true;
                } else if (mediaAsset != null && mediaAsset.includeInProofOfPlay) {
                    includeInPoP = true;
                }

                //logger.debug("PoP: Include in Proof of Play: " + includeInPoP);

                // Build the string
                if (includeInPoP) {
                    if (mediaAsset != null) {
                        logString = mediaAsset.externalId + ",";
                    } else {
                        logString = ",";
                    }

                    if (layout != null) {
                        logString = logString + layout.externalId + ",";
                    } else {
                        logString = logString + ",";
                    }

                    if (mediaWindow != null) {
                        logString = logString + mediaWindow.externalId + ",";
                    } else {
                        logString = logString + ",";
                    }

                    if (playlist != null) {
                        logString = logString + playlist.externalId + ",";
                    } else {
                        logString = logString + ",";
                    }

                    if (playlistMediaAsset != null) {
                        logString = logString + playlistMediaAsset.externalId + ",";
                    } else {
                        logString = logString + ",";
                    }

                    logString = logString + start + "," + end + "," + playReason + "," + endReason;
                    var popAssetID = null;

                    //Add pop_asset_id
                    if (playlistMediaAsset != null && playlistMediaAsset.popAssetID != null && playlistMediaAsset.popAssetID != "") {
                        popAssetID = playlistMediaAsset.popAssetID;
                    } else if (mediaAsset != null && mediaAsset.popAssetID != null) {
                        popAssetID = mediaAsset.popAssetID;
                    } else {
                        popAssetID = "";
                    }
                    logString = logString + "," + popAssetID;

                    msg = "PoP - Log - " + logString;
                    logger.info(msg);

                    File.append(Settings.application.fileSystem.systemFiles.proofOfPlayFilePath, logString + "\r\n", "text/csv", true);

                } else {
                    logger.debug("Asset is not included in Proof of Play");
                }
            } else if (this.config == null) {
                logger.debug("Proof of Play Config is null");
            } else if (this.config.enabled == false) {
                logger.debug("Proof of Play is not enabled");
            }
        }
    };

    window.proofOfPlayLogger = proofOfPlayLogger;

    return proofOfPlayLogger;
});